@@ -21,14 +21,14 @@
 # ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 #
 use strict;
+#use diagnostics;
 use warnings;
+no warnings 'uninitialized';
 use 5.012;
 use JSON::RPC::Client;
 use Term::ShellUI;
-use Data::Dump qw[dump];
+use Data::Dumper;
 use Getopt::Long;
-use utf8;
-binmode STDOUT, ":utf8";
 use Env qw[$XDG_CONFIG_HOME $HOME];
 
 # use non-standart paths
@@ -41,17 +41,23 @@ BEGIN {
 }
 
 # preparing terminal
+use utf8;
+use locale;
 binmode STDOUT, ':utf8';
 
 # configuration
 our %config;
-$config{version}=0.1;
+$config{version}=0.4;
+$config{revision}=20130518;
 require "cli-jsonrpc-config.pl";
-my $version,my $help;
-GetOptions ('V|version' => \$version, 'h|help' => \$help);
+
+# processing command line options
+my $version,my $help, my $command;
+GetOptions ('V|version' => \$version, 'h|help' => \$help, 'c=s' => \$command) or die("Invalid options");
 if ($version)
 {
-	print("Command line JSON-RPC interface version: $config{version}\n"); exit(1);
+	print("Command line JSON-RPC interface version.revision: $config{version}.$config{revision}\n");
+	exit(1);
 }
 if ($help)
 {
@@ -63,13 +69,17 @@ EiskaltDC++ is a cross-platform program that uses the Direct Connect and ADC pro
 
 Keys:
 \t-h, --help\t Show this message
-\t-V, --version\t Show version string\n");
+\t-V, --version\t Show version string
+\t-c command\t Execute jsonrpc method and exit\n");
 	exit(1);
 }
-print("Configuration:\n");
-foreach (keys %config)
+if ($config{debug} > 0)
 {
-	print("$_: $config{$_}\n");
+	print("Configuration:\n");
+	foreach (keys %config)
+	{
+		print("$_: $config{$_}\n");
+	}
 }
 
 # rest variables
@@ -77,14 +87,25 @@ my $obj;
 $obj->{'jsonrpc'} = $config{jsonrpc};
 my $res;
 
-# creating and configuring client
+# creating and configuring jsonrpc client
 my $client = new JSON::RPC::Client;
 $client->version("2.0");
 $client->ua->timeout(10);
 #$client->ua->credentials('http://127.0.0.1:3121', 'jsonrpc', 'user' => 'password');
+#$client->ua()->ssl_opts(verify_hostname=>0);
 
-# creating shell
+# preparing shell: reading command list, configuring history file and so
 my $term = new Term::ShellUI(commands => get_commands(), history_file => $config{hist_file}, history_max => $config{hist_max});
+
+# execute command if -c option used end exit 
+if (defined($command))
+{
+	if ($config{debug} > 0) { print("Request: $command\n") };
+	$term->process_a_cmd($command);
+	exit(0);
+}
+
+# starting a shell if no -c option used
 $term->prompt("$config{prompt}");
 $term->run();
 
@@ -146,6 +167,25 @@ sub get_commands
 			maxargs => 1,
 			proc => \&hublist
 		},
+		"hub.listfulldesc" =>
+		{
+			desc => "Show detailed list of connected hubs. Parameters: none",
+			proc => \&hublistfulldesc
+		}, 
+		"hub.getusers" =>
+		{
+			desc => "Show list of connected users for given hub. Parameters: huburl",
+			minargs => 1,
+			maxargs => 1,
+			proc => \&hubgetusers
+		},
+		"hub.getuserinfo" =>
+		{
+			desc => "Show detailed information for given user nick and hub. Parameters: nick, huburl",
+			minargs => 2,
+			maxargs => 2,
+			proc => \&hubgetuserinfo
+		},
 		"share.add" =>
 		{
 			desc => "Add a directory to share as virtual name. Parameters: directory, virtual name",
@@ -220,21 +260,21 @@ sub get_commands
 		},
 		"queue.setpriority" =>
 		{
-			desc => "Set download queue priority. Parameners: target, proirity",
+			desc => "Set download queue priority. Parameters: target, proirity. Priority is an integer from 0(paused) to 3(high)",
 			minargs => 2,
 			maxargs => 2,
 			proc => \&qsetprio
 		},
 		"queue.move" =>
 		{
-			desc => "Move queue item from source to target. Parameners: source, target",
+			desc => "Move queue item from source to target. Parameters: source, target",
 			minargs => 2,
 			maxargs => 2,
 			proc => \&qmove
 		},
 		"queue.remove" =>
 		{
-			desc => "Delete queue item by target. Parameners: target",
+			desc => "Delete queue item by target. Parameters: target",
 			minargs => 1,
 			maxargs => 1,
 			proc => \&qremove
@@ -255,6 +295,33 @@ sub get_commands
 			maxargs => 1,
 			proc => \&searchclear
 		},
+		"queue.getsources" =>
+		{
+			desc => "Show sources for specified target. Parameters: target, separator",
+			minargs => 1,
+			maxargs => 2,
+			proc => \&qgetsources
+		},
+		"hash.status" =>
+		{
+			desc => "Show hashing process status. Parameters: none",
+			proc => \&hashstatus
+		},
+		"hash.pause" =>
+		{
+			desc => "Pause hashing process. Parameters: none",
+			proc => \&hashpause
+		},
+		"methods.list" =>
+		{
+			desc => "List all jsonrpc methods available. Parameters: none",
+			proc => \&methodslist
+		},
+		"queue.matchlists" =>
+		{
+			desc => "Description here. Parameters: none",
+			proc => \&qmatchlists
+		},
 		# last
 		"prompt" =>
 		{
@@ -297,17 +364,17 @@ sub magnetadd($$)
 	$obj->{'method'} = 'magnet.add';
 	$obj->{'params'}->{'magnet'}=$_[0];
 	$obj->{'params'}->{'directory'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -321,17 +388,17 @@ sub daemonstop()
 {
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'daemon.stop';
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -346,17 +413,17 @@ sub hubadd($$)
 	$obj->{'method'} = 'hub.add';
 	$obj->{'params'}->{'huburl'}=$_[0];
 	$obj->{'params'}->{'enc'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -371,17 +438,17 @@ sub hubdel($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'hub.del';
 	$obj->{'params'}->{'huburl'}=$_[0];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -397,17 +464,17 @@ sub hubsay($$)
 	$obj->{'method'} = 'hub.say';
 	$obj->{'params'}->{'huburl'}=$_[0];
 	$obj->{'params'}->{'message'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -424,17 +491,17 @@ sub hubpm($$$)
 	$obj->{'params'}->{'huburl'}=$_[0];
 	$obj->{'params'}->{'nick'}=$_[1];
 	$obj->{'params'}->{'message'}=$_[2];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -449,17 +516,122 @@ sub hublist($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'hub.list';
 	$obj->{'params'}->{'separator'}=($_[0] || $config{'separator'});
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+	delete($obj->{'params'});
+}
+
+sub hublistfulldesc($)
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'hub.listfulldesc';
+	$obj->{'params'}->{'separator'}=($_[0] || $config{'separator'});
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n");
+			if (defined($res->result))
+			{
+				for my $key (keys %{$res->result})
+				{
+					print("$key:\n");
+					foreach ($res->result->{$key})
+					{
+						print("Total share:\t\t".$_->{totalshare}."\n");
+						print("Connected:\t\t".$_->{connected}."\n");
+						print("Users:\t\t".$_->{users}."\n");
+						print("Description:\t\t".$_->{description}."\n");
+						print("Hub name:\t\t".$_->{hubname}."\n");
+						
+					}
+				}
+			}
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+	delete($obj->{'params'});
+}
+
+
+sub hubgetusers()
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'hub.getusers';
+	$obj->{'params'}->{'huburl'}=$_[0];
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n");
+			print(join("\n",split(/;/,$res->result))."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+	delete($obj->{'params'});
+}
+
+sub hubgetuserinfo()
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'hub.getuserinfo';
+	$obj->{'params'}->{'nick'}=$_[0];
+	$obj->{'params'}->{'huburl'}=$_[1];
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n");
+			if (defined($res->result))
+			{
+				#for my $key (keys %{$res->result})
+				#{
+				#	print("$key:\t\t$res->result->{$key}\n");
+				#}
+				while ( my ($key, $value) = each(%{$res->result}) )
+				{
+					print "$key:\t\t$value\n";
+				}
+			}
 		}
 	}
 	else
@@ -475,17 +647,17 @@ sub shareadd($$)
 	$obj->{'method'} = 'share.add';
 	$obj->{'params'}->{'directory'}=$_[0];
 	$obj->{'params'}->{'virtname'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -501,17 +673,17 @@ sub sharerename($$)
 	$obj->{'method'} = 'share.rename';
 	$obj->{'params'}->{'directory'}=$_[0];
 	$obj->{'params'}->{'virtname'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -526,17 +698,17 @@ sub sharedel($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'share.del';
 	$obj->{'params'}->{'directory'}=$_[0];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -551,17 +723,17 @@ sub sharelist($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'share.list';
 	$obj->{'params'}->{'separator'}=($_[0] || $config{'separator'});
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -575,17 +747,17 @@ sub sharerefresh()
 {
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'share.refresh';
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -600,17 +772,17 @@ sub listdownload($$)
 	$obj->{'method'} = 'list.download';
 	$obj->{'params'}->{'huburl'}=$_[0];
 	$obj->{'params'}->{'nick'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -626,17 +798,17 @@ sub hubgetchat($$)
 	$obj->{'method'} = 'hub.getchat';
 	$obj->{'params'}->{'huburl'}=$_[0];
 	$obj->{'params'}->{'separator'}=($_[1] || $config{'separator'});
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -651,17 +823,17 @@ sub searchsend($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'search.send';
 	$obj->{'params'}->{'searchstring'}=$_[0];
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -676,17 +848,39 @@ sub searchgetresults($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'search.getresults';
 	if (defined($_[0])) {$obj->{'params'}->{'huburl'}=$_[0]};
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n");
+			foreach (@{$res->result})
+			{
+				print("CID:\t\t".$_->{CID}."\n");
+				print("Connection:\t\t".$_->{Connection}."\n");
+				print("Exact Size:\t\t".$_->{'Exact Size'}."\n");
+				print("File Order:\t\t".$_->{'File Order'}."\n");
+				print("Filename:\t\t".$_->{Filename}."\n");
+				print("Free Slots:\t\t".$_->{'Free Slots'}."\n");
+				print("Hub:\t\t".$_->{Hub}."\n");
+				print("Hub URL:\t\t".$_->{'Hub URL'}."\n");
+				print("Icon:\t\t".$_->{Icon}."\n");
+				print("IP:\t\t".$_->{IP}."\n");
+				print("Nick:\t\t".$_->{Nick}."\n");
+				print("Path:\t\t".$_->{Path}."\n");
+				print("Real Size:\t\t".$_->{'Real Size'}."\n");
+				print("Shared:\t\t".$_->{Shared}."\n");
+				print("Size:\t\t".$_->{Size}."\n");
+				print("Slots:\t\t".$_->{Slots}."\n");
+				print("Slots Order:\t\t".$_->{'Slots Order'}."\n");
+				print("TTH:\t\t".$_->{TTH}."\n");
+				print("Type:\t\t".$_->{Type}."\n\n");
+			}
 		}
 	}
 	else
@@ -701,17 +895,17 @@ sub searchclear($)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'search.clear';
 	if (defined($_[0])) {$obj->{'params'}->{'huburl'}=$_[0]};
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -725,17 +919,17 @@ sub showversion()
 {
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'show.version';
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -748,21 +942,20 @@ sub showratio()
 {
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'show.ratio';
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			#utf8::encode($res->content->{up});
 			print("===Reply===\n");
-			print("Up:\t".dump($res->content->{up})."\n");
-			print("Down:\t".dump($res->content->{down})."\n");
-			print("Ratio:\t".dump($res->content->{ratio})."\n");
+			print("Ratio:\t\t".$res->result->{ratio}."\n");
+			print("Upload:\t\t".$res->result->{up}."\n");
+			print("Download:\t".$res->result->{down}."\n");
 		}
 	}
 	else
@@ -776,18 +969,18 @@ sub qsetprio($$)
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'queue.setpriority';
 	$obj->{'params'}->{'target'}=$_[0];
-	$obj->{'params'}->{'priority'}=$_[1];
-	print("===Request===\n".dump($obj)."\n");
+	$obj->{'params'}->{'priority'}=$_[1]+0;
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -797,31 +990,97 @@ sub qsetprio($$)
 	delete($obj->{'params'});
 }
 
-sub qmove()
+sub qmove($$)
 {
-
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'queue.move';
+	$obj->{'params'}->{'source'}=$_[0];
+	$obj->{'params'}->{'target'}=$_[1];
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n".$res->result."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+	delete($obj->{'params'});
 }
 
-sub qremove()
+sub qremove($)
 {
-
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'queue.remove';
+	$obj->{'params'}->{'target'}=$_[0];
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n".$res->result."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+	delete($obj->{'params'});
 }
 
 sub qlist()
 {
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'queue.list';
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n");
+			if (defined($res->result))
+			{
+				for my $key (keys %{$res->result})
+				{
+					print("$key:\n");
+					foreach ($res->result->{$key})
+					{
+						print("Added:\t\t".$_->{Added}."\n");
+						print("Downloaded:\t\t".$_->{Downloaded}."\n");
+						print("Downloaded Sort:\t\t".$_->{'Downloaded Sort'}."\n");
+						print("Errors:\t\t".$_->{Errors}."\n");
+						print("Exact Size:\t\t".$_->{'Exact Size'}."\n");
+						print("Filename:\t\t".$_->{Filename}."\n");
+						print("Path:\t\t".$_->{Path}."\n");
+						print("Priority:\t\t".$_->{Priority}."\n");
+						print("Size:\t\t".$_->{Size}."\n");
+						print("Size Sort:\t\t".$_->{'Size Sort'}."\n");
+						print("Status:\t\t".$_->{Status}."\n");
+						print("Target:\t\t".$_->{Target}."\n");
+						print("TTH:\t\t".$_->{TTH}."\n");
+						print("Users:\t\t".$_->{Users}."\n\n");
+					}
+				}
+			}
+			else {print("No files in queue\n");}
 		}
 	}
 	else
@@ -834,17 +1093,142 @@ sub qlisttargets()
 {
 	$obj->{'id'} = int(rand(2**16));
 	$obj->{'method'} = 'queue.listtargets';
-	print("===Request===\n".dump($obj)."\n");
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
 	$res = $client->call($config{eiskaltURL}, $obj);
 	if ($res)
 	{
 		if ($res->is_error) 
 		{
-			print("===Error===\n".dump($res->error_message)."\n");
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
 		}
 		else
 		{
-			print("===Reply===\n".dump($res->result)."\n");
+			print("===Reply===\n".$res->result."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+}
+
+sub qgetsources($$)
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'queue.getsources';
+	$obj->{'params'}->{'target'}=$_[0];
+	$obj->{'params'}->{'separator'}=($_[1] || $config{'separator'});
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n");
+			print("Online:\t\t".$res->result->{online}."\n");
+			print("Sources:\t\t".$res->result->{sources}."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+	delete($obj->{'params'});
+}
+
+sub hashstatus()
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'hash.status';
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n");
+			print("Bytesleft:\t\t".$res->result->{bytesleft}."\n");
+			print("Currentfile:\t\t".$res->result->{currentfile}."\n");
+			print("Filesleft:\t\t".$res->result->{filesleft}."\n");
+			print("Status:\t\t".$res->result->{status}."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+}
+
+sub hashpause()
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'hash.pause';
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n".$res->result."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+}
+
+sub methodslist()
+{
+	use JSON;
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'methods.list';
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n")};
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n".$res->result."\n");
+		}
+	}
+	else
+	{
+		print $client->status_line;
+	}
+}
+
+sub qmatchlists()
+{
+	$obj->{'id'} = int(rand(2**16));
+	$obj->{'method'} = 'queue.matchlists';
+	if ($config{debug} > 0) { print("===Request===\n".Dumper($obj)."\n") };
+	$res = $client->call($config{eiskaltURL}, $obj);
+	if ($res)
+	{
+		if ($res->is_error) 
+		{
+			print("===Error===\nCode: ".$res->error_message->{'code'}."===\n".$res->error_message->{'message'}."\n");
+		}
+		else
+		{
+			print("===Reply===\n".$res->result."\n");
 		}
 	}
 	else
@@ -858,34 +1242,40 @@ __END__
 
 =pod
 
-# known methods
-std::string("magnet.add")))
-std::string("daemon.stop")))
-std::string("hub.add")))
-std::string("hub.del")))
-std::string("hub.say")))
-std::string("hub.pm")))
-std::string("hub.list")))
-std::string("hub.getchat")))
-std::string("share.add")))
-std::string("share.rename")))
-std::string("share.del")))
-std::string("share.list")))
-std::string("share.refresh")))
-std::string("list.download")))
-std::string("search.send")))
-std::string("search.getresults")))
-std::string("search.clear")))
-std::string("show.version")))
-std::string("show.ratio")))
-std::string("queue.setpriority")))
-std::string("queue.move")))
-std::string("queue.remove")))
-std::string("queue.listtargets")))
-std::string("queue.list")))
-std::string("queue.getsources")))
-std::string("hash.status")))
-std::string("hash.pause")))
-std::string("methods.list")))
+# known methods as for 0.2.17062012
+# grep "AddMethod" eiskaltdcpp-daemon/ServerThread.cpp | egrep -o "std::string\(.*\)"
+magnet.add +0.3
+daemon.stop +0.3
+hub.add +0.3
+hub.del +0.3
+hub.say +0.3
+hub.pm +0.3
+hub.list +0.3
+hub.getchat +0.3
+share.add +0.3
+share.rename +0.3
+share.del +0.3
+share.list +0.3
+share.refresh +0.3
+list.download +0.3
+search.send +0.3
+search.getresults +0.3
+search.clear +0.3
+show.version +0.3
+show.ratio +0.3
+queue.setpriority -
+queue.move -
+queue.remove +0.3
+queue.listtargets +0.3
+queue.list +0.3
+queue.getsources +0.3
+hash.status +0.3
+hash.pause +0.3
+methods.list +0.3
+queue.matchlists +0.3
+hub.listfulldesc +0.4
+hub.getusers
+hub.getuserinfo
 
 =cut
+
