@@ -17,6 +17,7 @@
 
 #include "dcpp/ClientManager.h"
 #include "dcpp/Client.h"
+#include "dcpp/AdcHub.h"
 #include "dcpp/ConnectionManager.h"
 #include "dcpp/FavoriteManager.h"
 #include "dcpp/DownloadManager.h"
@@ -113,6 +114,10 @@ int ServerThread::run() {
     xmlrpc_c::methodPtr const getHashStatusMethodP(new getHashStatusMethod);
     xmlrpc_c::methodPtr const pauseHashMethodP(new pauseHashMethod);
     xmlrpc_c::methodPtr const getMethodListMethodP(new getMethodListMethod);
+    xmlrpc_c::methodPtr const listHubsFullDescMethodP(new listHubsFullDescMethod);
+    xmlrpc_c::methodPtr const getHubUserListMethodP(new getHubUserListMethod);
+    xmlrpc_c::methodPtr const getUserInfoMethodP(new getUserInfoMethod);
+    xmlrpc_c::methodPtr const matchAllListMethodP(new matchAllListMethod);
     xmlrpcRegistry.addMethod("magnet.add", magnetAddMethodP);
     xmlrpcRegistry.addMethod("daemon.stop", stopDaemonMethodP);
     xmlrpcRegistry.addMethod("hub.add", hubAddMethodP);
@@ -141,6 +146,10 @@ int ServerThread::run() {
     xmlrpcRegistry.addMethod("hash.status", getHashStatusMethodP);
     xmlrpcRegistry.addMethod("hash.pause", pauseHashMethodP);
     xmlrpcRegistry.addMethod("method.list", getMethodListMethodP);
+    xmlrpcRegistry.addMethod("hub.listfulldesc", listHubsFullDescMethodP);
+    xmlrpcRegistry.addMethod("hub.getusers", getHubUserListMethodP);
+    xmlrpcRegistry.addMethod("hub.getuserinfo", getUserInfoMethodP);
+    xmlrpcRegistry.addMethod("queue.matchlists", matchAllListMethodP);
     xmlrpcRegistry.setShutdown(new systemShutdownMethod);
     sock.create();
     sock.setSocketOpt(SO_REUSEADDR, 1);
@@ -158,38 +167,42 @@ int ServerThread::run() {
 #ifdef JSONRPC_DAEMON
     jsonserver = new Json::Rpc::HTTPServer(lip, lport);
     JsonRpcMethods a;
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::MagnetAdd, std::string("magnet.add"), a.GetDescriptionMagnetAdd()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::StopDaemon, std::string("daemon.stop"), a.GetDescriptionStopDaemon()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubAdd, std::string("hub.add"), a.GetDescriptionHubAdd()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubDel, std::string("hub.del"), a.GetDescriptionHubDel()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubSay, std::string("hub.say"), a.GetDescriptionHubSay()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubSayPM, std::string("hub.pm"), a.GetDescriptionHubSayPM()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListHubs, std::string("hub.list"), a.GetDescriptionListHubs()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetChatPub, std::string("hub.getchat"), a.GetDescriptionGetChatPub()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::AddDirInShare, std::string("share.add"), a.GetDescriptionAddDirInShare()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::RenameDirInShare, std::string("share.rename"), a.GetDescriptionRenameDirInShare()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::DelDirFromShare, std::string("share.del"), a.GetDescriptionDelDirFromShare()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListShare, std::string("share.list"), a.GetDescriptionListShare()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::RefreshShare, std::string("share.refresh"), a.GetDescriptionRefreshShare()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetFileList, std::string("list.download"), a.GetDescriptionGetFileList()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::SendSearch, std::string("search.send"), a.GetDescriptionSendSearch()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ReturnSearchResults, std::string("search.getresults"), a.GetDescriptionReturnSearchResults()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ClearSearchResults, std::string("search.clear"), a.GetDescriptionClearSearchResults()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ShowVersion, std::string("show.version"), a.GetDescriptionShowVersion()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ShowRatio, std::string("show.ratio"), a.GetDescriptionShowRatio()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::SetPriorityQueueItem, std::string("queue.setpriority"), a.GetDescriptionSetPriorityQueueItem()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::MoveQueueItem, std::string("queue.move"), a.GetDescriptionMoveQueueItem()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::RemoveQueueItem, std::string("queue.remove"), a.GetDescriptionRemoveQueueItem()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListQueueTargets, std::string("queue.listtargets"), a.GetDescriptionListQueueTargets()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListQueue, std::string("queue.list"), a.GetDescriptionListQueue()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetSourcesItem, std::string("queue.getsources"), a.GetDescriptionGetSourcesItem()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetHashStatus, std::string("hash.status"), a.GetDescriptionGetHashStatus()));
-    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::PauseHash, std::string("hash.pause"), a.GetDescriptionPauseHash()));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::MagnetAdd, std::string("magnet.add")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::StopDaemon, std::string("daemon.stop")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubAdd, std::string("hub.add")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubDel, std::string("hub.del")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubSay, std::string("hub.say")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::HubSayPM, std::string("hub.pm")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListHubs, std::string("hub.list")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListHubsFullDesc, std::string("hub.listfulldesc")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetChatPub, std::string("hub.getchat")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::AddDirInShare, std::string("share.add")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::RenameDirInShare, std::string("share.rename")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::DelDirFromShare, std::string("share.del")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListShare, std::string("share.list")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::RefreshShare, std::string("share.refresh")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetFileList, std::string("list.download")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::SendSearch, std::string("search.send")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ReturnSearchResults, std::string("search.getresults")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ClearSearchResults, std::string("search.clear")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ShowVersion, std::string("show.version")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ShowRatio, std::string("show.ratio")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::SetPriorityQueueItem, std::string("queue.setpriority")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::MoveQueueItem, std::string("queue.move")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::RemoveQueueItem, std::string("queue.remove")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListQueueTargets, std::string("queue.listtargets")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::ListQueue, std::string("queue.list")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetSourcesItem, std::string("queue.getsources")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetHashStatus, std::string("hash.status")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::PauseHash, std::string("hash.pause")));
     jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetMethodList, std::string("methods.list")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::MatchAllLists, std::string("queue.matchlists")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetHubUserList, std::string("hub.getusers")));
+    jsonserver->AddMethod(new Json::Rpc::RpcMethod<JsonRpcMethods>(a, &JsonRpcMethods::GetUserInfo, std::string("hub.getuserinfo")));
 
     if (!jsonserver->startPolling())
         std::cout << "JSONRPC: Start mongoose failed" << std::endl;
-    else 
+    else
         std::cout << "JSONRPC: Start mongoose" << std::endl;
 #endif
 
@@ -272,7 +285,7 @@ void ServerThread::disconnectClient(const string& address) {
 }
 
 void ServerThread::on(TimerManagerListener::Second, uint64_t aTick) noexcept {
-    //int64_t diff = (int64_t)((lastUpdate == 0) ? aTick - 1000 : aTick - lastUpdate);
+
     int64_t upDiff = Socket::getTotalUp() - lastUp;
     int64_t downDiff = Socket::getTotalDown() - lastDown;
 
@@ -288,9 +301,7 @@ void ServerThread::on(TimerManagerListener::Second, uint64_t aTick) noexcept {
 void ServerThread::on(Connecting, Client* cur) noexcept {
     if (isVerbose)
         cout << "Connecting to " <<  cur->getHubUrl() << "..."<< "\n";
-}
 
-void ServerThread::on(Connected, Client* cur) noexcept {
     ClientIter i = clientsMap.find(cur->getHubUrl());
     if (i == clientsMap.end()) {
         CurHub curhub;
@@ -298,18 +309,43 @@ void ServerThread::on(Connected, Client* cur) noexcept {
         clientsMap[cur->getHubUrl()] = curhub;
     } else if (i != clientsMap.end() && clientsMap[cur->getHubUrl()].curclient == NULL)
         clientsMap[cur->getHubUrl()].curclient = cur;
+}
 
+void ServerThread::on(Connected, Client* cur) noexcept {
     if (isVerbose)
-        cout << "Connecting to " << cur->getHubUrl() << "..." << endl;
+        cout << "Connected to " << cur->getHubUrl() << "..." << endl;
 }
 
-void ServerThread::on(UserUpdated, Client*, const OnlineUserPtr& user) noexcept {
+void ServerThread::on(UserUpdated, Client* cur, const OnlineUser& user) noexcept {
+    Identity id = user.getIdentity();
+
+    if (!id.isHidden())
+    {
+        StringMap params;
+        getParamsUser(params, id);
+        if (isDebug) {printf ("HUB: %s == UserUpdated %s\n", cur->getHubUrl().c_str(), params["Nick"].c_str()); fflush (stdout);}
+        updateUser(params, cur);
+    }
+}
+void ServerThread::on(UsersUpdated, Client* cur, const OnlineUserList& list) noexcept {
+    Identity id;
+
+    for (auto it = list.begin(); it != list.end(); ++it)
+    {
+        id = (*it)->getIdentity();
+        if (!id.isHidden())
+        {
+            StringMap params;
+            getParamsUser(params, id);
+            if (isDebug) {printf ("HUB: %s == UsersUpdated %s\n", cur->getHubUrl().c_str(), params["Nick"].c_str()); fflush (stdout);}
+            updateUser(params, cur);
+        }
+    }
+
 }
 
-void ServerThread::on(UsersUpdated, Client*, const OnlineUserList& aList) noexcept {
-}
-
-void ServerThread::on(UserRemoved, Client*, const OnlineUserPtr& user) noexcept {
+void ServerThread::on(UserRemoved, Client* cur, const OnlineUser& user) noexcept {
+    removeUser(user.getUser()->getCID().toBase32(), cur);
 }
 
 void ServerThread::on(Redirect, Client* cur, const string& line) noexcept {
@@ -419,8 +455,7 @@ void ServerThread::startSocket(bool changed) {
 }
 
 void ServerThread::showPortsError(const string& port) {
-    fprintf(stdout,
-            "\n\t\tConnectivity Manager: Warning\n\n Unable to open port %s. "
+    printf("\n\t\tConnectivity Manager: Warning\n\n Unable to open port %s. "
             "Searching or file transfers will\n not work correctly "
             "until you change settings or turn off\n any application "
             "that might be using that port.\n\n",
@@ -434,7 +469,6 @@ void ServerThread::sendMessage(const string& hubUrl, const string& message) {
         Client* client = i->second.curclient;
         if (client && !message.empty()) {
             bool thirdPerson = !message.compare(0, 3, "/me");
-            //printf("%s\t%s\n'", message.c_str(), message.substr(4).c_str());
             client->hubMessage(thirdPerson ? message.substr(4) : message , thirdPerson);
         }
     }
@@ -454,21 +488,21 @@ bool ServerThread::findHubInConnectedClients(const string& hub) {
     return i != clientsMap.end();
 }
 
-string ServerThread::sendPrivateMessage(const string& hub, const string& nick, const string& message) {
+bool ServerThread::sendPrivateMessage(const string& hub, const string& nick, const string& message) {
     ClientIter i = clientsMap.find(hub);
     if (i != clientsMap.end() && clientsMap[i->first].curclient != NULL) {
         Client* client = i->second.curclient;
         if (client && !message.empty()) {
             bool thirdPerson = !message.compare(0, 3, "/me");
-            UserPtr user = ClientManager::getInstance()->getUser(nick, hub);
-            if (user && user->isOnline())
-            {
+            auto it = i->second.curuserlist.find(nick);
+            if (it == i->second.curuserlist.end())
+                return false;
+            UserPtr user = ClientManager::getInstance()->findUser(CID(it->second));
+            if (user && user->isOnline()) {
                 ClientManager::getInstance()->privateMessage(HintedUser(user, hub), thirdPerson ? message.substr(4) : message, thirdPerson);
-                return "Private message sent to " + nick + " at " + hub;
-            }
-            else
-            {
-                return "User went offline at " + hub;
+                return true;
+            } else {
+                return false;
             }
         }
     }
@@ -483,7 +517,7 @@ bool ServerThread::getFileList(const string& hub, const string& nick, bool match
             try {
                 //UserPtr user = ClientManager::getInstance()->findUser(CID(cid));
                 UserPtr user = ClientManager::getInstance()->getUser(nick, hub);
-                if (user) {
+                if (user && user->isOnline() && user->isNMDC()) {
                     const HintedUser hintedUser(user, i->first);
                     if (user == ClientManager::getInstance()->getMe()) {
                         // Don't download file list, open locally instead
@@ -719,10 +753,18 @@ bool ServerThread::addInQueue(const string& sddir, const string& name, const int
     if (name.empty() && tth.empty())
         return false;
 
-    if (sddir.empty())
-        QueueManager::getInstance()->add(SETTING(DOWNLOAD_DIRECTORY) + PATH_SEPARATOR_STR + name, size, TTHValue(tth));
-    else
-        QueueManager::getInstance()->add(sddir + PATH_SEPARATOR_STR + name, size, TTHValue(tth));
+    try
+    {
+        if (sddir.empty())
+            QueueManager::getInstance()->add(SETTING(DOWNLOAD_DIRECTORY) + PATH_SEPARATOR_STR + name, size, TTHValue(tth));
+        else
+            QueueManager::getInstance()->add(sddir + PATH_SEPARATOR_STR + name, size, TTHValue(tth));
+    }
+    catch (const Exception& e)
+    {
+        if (isDebug) std::cout << "ServerThread::addInQueue->(" << e.getError() << ")"<< std::endl;
+        return false;
+    }
 
     return true;
 }
@@ -763,122 +805,106 @@ void ServerThread::getItemSources(QueueItem* item, const string& separator, stri
     for (QueueItem::SourceConstIter it = item->getSources().begin(); it != item->getSources().end(); ++it) {
         if (it->getUser().user->isOnline())
             ++online_tmp;
-        //printf("++online_tmp: %u\n", online_tmp);fflush(stdout);
         if (!sources.empty())
             sources += separator;
         nick = Util::toString(ClientManager::getInstance()->getNicks(it->getUser().user->getCID(), it->getUser().hint));
         sources += nick;
     }
-    //printf("online_tmp: %u\n", online_tmp);fflush(stdout);
 }
 void ServerThread::getItemSourcesbyTarget(const string& target, const string& separator, string& sources, unsigned int& online) {
     const QueueItem::StringMap &ll = QueueManager::getInstance()->lockQueue();
     for (auto it = ll.begin(); it != ll.end(); ++it) {
-        //printf("target: %s\n *it->first: %s\n", target.c_str(),(*it->first).c_str());fflush(stdout);
         if (target == *it->first) {
             getItemSources(it->second, separator, sources, online);
-            //printf("online1: %u\n", online);fflush(stdout);
         }
-        //printf("online2: %u\n", online);fflush(stdout);
     }
     QueueManager::getInstance()->unlockQueue();
 }
 
 void ServerThread::getQueueParams(QueueItem* item, StringMap& params) {
-	string nick;
-	unsigned int online = 0;
+    string nick;
+    unsigned int online = 0;
 
-	params["Filename"] = item->getTargetFileName();
-	params["Path"] = Util::getFilePath(item->getTarget());
-	params["Target"] = item->getTarget();
+    params["Filename"] = item->getTargetFileName();
+    params["Path"] = Util::getFilePath(item->getTarget());
+    params["Target"] = item->getTarget();
 
-	params["Users"] = "";
-    
-	//for (QueueItem::SourceConstIter it = item->getSources().begin(); it != item->getSources().end(); ++it) {
-		//if (it->getUser().user->isOnline())
-			//++online;
+    params["Users"] = "";
 
-		//if (params["Users"].size() > 0)
-			//params["Users"] += ", ";
-
-		//nick = Util::toString(ClientManager::getInstance()->getNicks(it->getUser().user->getCID(), it->getUser().hint));
-		//params["Users"] += nick;
-	//}
-    
     getItemSources(item, ", ", params["Users"], online);
-    
-	if (params["Users"].empty())
-		params["Users"] = _("No users");
 
-	// Status
-	if (item->isWaiting())
-		params["Status"] = Util::toString(online) + _(" of ") + Util::toString(item->getSources().size()) + _(" user(s) online");
-	else
-		params["Status"] = _("Running...");
+    if (params["Users"].empty())
+        params["Users"] = _("No users");
 
-	// Size
-	params["Size Sort"] = Util::toString(item->getSize());
-	if (item->getSize() < 0) {
-		params["Size"] = _("Unknown");
-		params["Exact Size"] = _("Unknown");
-	} else {
-		params["Size"] = Util::formatBytes(item->getSize());
-		params["Exact Size"] = Util::formatExactSize(item->getSize());
-	}
+    // Status
+    if (item->isWaiting())
+        params["Status"] = Util::toString(online) + _(" of ") + Util::toString(item->getSources().size()) + _(" user(s) online");
+    else
+        params["Status"] = _("Running...");
 
-	// Downloaded
-	params["Downloaded Sort"] = Util::toString(item->getDownloadedBytes());
-	if (item->getSize() > 0) {
-		double percent = (double)item->getDownloadedBytes() * 100.0 / (double)item->getSize();
-		params["Downloaded"] = Util::formatBytes(item->getDownloadedBytes()) + " (" + Util::toString(percent) + "%)";
-	} else {
-		params["Downloaded"] = _("0 B (0.00%)");
-	}
+    // Size
+    params["Size Sort"] = Util::toString(item->getSize());
+    if (item->getSize() < 0) {
+        params["Size"] = _("Unknown");
+        params["Exact Size"] = _("Unknown");
+    } else {
+        params["Size"] = Util::formatBytes(item->getSize());
+        params["Exact Size"] = Util::formatExactSize(item->getSize());
+    }
 
-	// Priority
-	switch (item->getPriority()) {
-		case QueueItem::PAUSED: params["Priority"] = _("Paused"); break;
-		case QueueItem::LOWEST: params["Priority"] = _("Lowest"); break;
-		case QueueItem::LOW: params["Priority"] = _("Low"); break;
-		case QueueItem::HIGH: params["Priority"] = _("High"); break;
-		case QueueItem::HIGHEST: params["Priority"] = _("Highest"); break;
-		default: params["Priority"] = _("Normal"); break;
-	}
+    // Downloaded
+    params["Downloaded Sort"] = Util::toString(item->getDownloadedBytes());
+    if (item->getSize() > 0) {
+        double percent = (double)item->getDownloadedBytes() * 100.0 / (double)item->getSize();
+        params["Downloaded"] = Util::formatBytes(item->getDownloadedBytes()) + " (" + Util::toString(percent) + "%)";
+    } else {
+        params["Downloaded"] = _("0 B (0.00%)");
+    }
 
-	// Error
-	params["Errors"] = "";
-	for (QueueItem::SourceConstIter it = item->getBadSources().begin(); it != item->getBadSources().end(); ++it) {
-		nick = Util::toString(ClientManager::getInstance()->getNicks(it->getUser().user->getCID(), it->getUser().hint));
+    // Priority
+    switch (item->getPriority()) {
+        case QueueItem::PAUSED: params["Priority"] = _("Paused"); break;
+        case QueueItem::LOWEST: params["Priority"] = _("Lowest"); break;
+        case QueueItem::LOW: params["Priority"] = _("Low"); break;
+        case QueueItem::HIGH: params["Priority"] = _("High"); break;
+        case QueueItem::HIGHEST: params["Priority"] = _("Highest"); break;
+        default: params["Priority"] = _("Normal"); break;
+    }
 
-		if (!it->isSet(QueueItem::Source::FLAG_REMOVED)) {
-			if (params["Errors"].size() > 0)
-				params["Errors"] += ", ";
-			params["Errors"] += nick + " (";
+    // Error
+    params["Errors"] = "";
+    for (QueueItem::SourceConstIter it = item->getBadSources().begin(); it != item->getBadSources().end(); ++it) {
+        nick = Util::toString(ClientManager::getInstance()->getNicks(it->getUser().user->getCID(), it->getUser().hint));
 
-			if (it->isSet(QueueItem::Source::FLAG_FILE_NOT_AVAILABLE))
-				params["Errors"] += _("File not available");
-			else if (it->isSet(QueueItem::Source::FLAG_PASSIVE))
-				params["Errors"] += _("Passive user");
-			else if (it->isSet(QueueItem::Source::FLAG_CRC_FAILED))
-				params["Errors"] += _("CRC32 inconsistency (SFV-Check)");
-			else if (it->isSet(QueueItem::Source::FLAG_BAD_TREE))
-				params["Errors"] += _("Full tree does not match TTH root");
-			else if (it->isSet(QueueItem::Source::FLAG_SLOW_SOURCE))
-				params["Errors"] += _("Source too slow");
-			else if (it->isSet(QueueItem::Source::FLAG_NO_TTHF))
-				params["Errors"] += _("Remote client does not fully support TTH - cannot download");
+        if (!it->isSet(QueueItem::Source::FLAG_REMOVED)) {
+            if (params["Errors"].size() > 0)
+                params["Errors"] += ", ";
+            params["Errors"] += nick + " (";
 
-			params["Errors"] += ")";
-		}
-	}
-	if (params["Errors"].empty())
-		params["Errors"] = _("No errors");
+            if (it->isSet(QueueItem::Source::FLAG_FILE_NOT_AVAILABLE))
+                params["Errors"] += _("File not available");
+            else if (it->isSet(QueueItem::Source::FLAG_PASSIVE))
+                params["Errors"] += _("Passive user");
+            else if (it->isSet(QueueItem::Source::FLAG_CRC_FAILED))
+                params["Errors"] += _("CRC32 inconsistency (SFV-Check)");
+            else if (it->isSet(QueueItem::Source::FLAG_BAD_TREE))
+                params["Errors"] += _("Full tree does not match TTH root");
+            else if (it->isSet(QueueItem::Source::FLAG_SLOW_SOURCE))
+                params["Errors"] += _("Source too slow");
+            else if (it->isSet(QueueItem::Source::FLAG_NO_TTHF))
+                params["Errors"] += _("Remote client does not fully support TTH - cannot download");
 
-	// Added
-	params["Added"] = Util::formatTime("%Y-%m-%d %H:%M", item->getAdded());
+            params["Errors"] += ")";
+        }
+    }
+    if (params["Errors"].empty())
+        params["Errors"] = _("No errors");
 
-	// TTH
-	params["TTH"] = item->getTTH().toBase32();
+    // Added
+    params["Added"] = Util::formatTime("%Y-%m-%d %H:%M", item->getAdded());
+
+    // TTH
+    params["TTH"] = item->getTTH().toBase32();
 }
 
 void ServerThread::listQueueTargets(string& listqueue, const string& sseparator) {
@@ -896,33 +922,44 @@ void ServerThread::listQueueTargets(string& listqueue, const string& sseparator)
     QueueManager::getInstance()->unlockQueue();
 }
 
-void ServerThread::updatelistQueueTargets() {
-    const QueueItem::StringMap &ll = QueueManager::getInstance()->lockQueue();
-    queuesMap.clear();
-    auto it = ll.begin(); unsigned int i = 0;
-    while (it != ll.end()) {
-        queuesMap[i] = *it->first;
-         ++it, ++i;
-    }
-    QueueManager::getInstance()->unlockQueue();
-}
+//void ServerThread::updatelistQueueTargets() {
+    //const QueueItem::StringMap &ll = QueueManager::getInstance()->lockQueue();
+    //queuesMap.clear();
+    //auto it = ll.begin(); unsigned int i = 0;
+    //while (it != ll.end()) {
+        //queuesMap[i] = *it->first;
+         //++it, ++i;
+    //}
+    //QueueManager::getInstance()->unlockQueue();
+//}
 
-void ServerThread::on(Added, QueueItem* item) noexcept {
-    queuesMap[queuesMap.size()+1] = item->getTarget();
+//void ServerThread::on(Added, QueueItem* item) noexcept {
+    //queuesMap[queuesMap.size()+1] = item->getTarget();
+//}
 
-}
+//void ServerThread::on(Finished, QueueItem*, const string&, int64_t) noexcept {
 
-void ServerThread::on(Finished, QueueItem*, const string&, int64_t) noexcept {
+//}
 
-}
+//void ServerThread::on(Removed, QueueItem* item) noexcept {
+    //auto it = queuesMap.begin();
+    //while (it != queuesMap.end()) {
+        //if (it->second == item->getTarget()) {
+            //it = queuesMap.erase(it);
+        //} else
+            //++it;
+    //}
+//}
 
-void ServerThread::on(Removed, QueueItem*) noexcept {
-
-}
-
-void ServerThread::on(Moved, QueueItem*, const string&) noexcept {
-
-}
+//void ServerThread::on(Moved, QueueItem* item, const string& oldTarget) noexcept {
+    //auto it = queuesMap.begin();
+    //while (it != queuesMap.end()) {
+        //if (it->second == oldTarget) {
+            //it->second = item->getTarget();
+        //} else
+            //++it;
+    //}
+//}
 
 void ServerThread::listQueue(unordered_map<string,StringMap>& listqueue) {
     const QueueItem::StringMap &ll = QueueManager::getInstance()->lockQueue();
@@ -934,6 +971,19 @@ void ServerThread::listQueue(unordered_map<string,StringMap>& listqueue) {
     QueueManager::getInstance()->unlockQueue();
 }
 
+void ServerThread::listHubsFullDesc(unordered_map<string,StringMap>& listhubs) {
+    for (ClientIter i = clientsMap.begin(); i != clientsMap.end(); ++i) {
+        Client* cl = i->second.curclient;
+        StringMap sm;
+        sm["connected"] = cl->isReady() ? "1"  : "0";
+        sm["users"] = Util::toString(cl->getUserCount());
+        sm["totalshare"] = Util::toString(cl->getAvailable());
+        sm["hubname"] = cl->getHubName();
+        sm["description"] = cl->getHubDescription();
+        listhubs[i->first] = sm;
+    }
+}
+
 bool ServerThread::moveQueueItem(const string& source, const string& target) {
     if (!source.empty() && !target.empty()) {
         if (target[target.length() - 1] == PATH_SEPARATOR) {
@@ -999,5 +1049,118 @@ bool ServerThread::pauseHash() {
 }
 
 void ServerThread::getMethodList(string& tmp) {
-    tmp = "magnet.add|daemon.stop|hub.add|hub.del|hub.say|hub.pm|hub.list|share.add|share.rename|share.del|share.list|share.refresh|list.download|hub.getchat|search.send|search.getresults|show.version|show.ratio|queue.setpriority|queue.move|queue.remove|queue.listtargets|queue.list|queue.getsources|hash.status|hash.pause|methods.list";
+    tmp = "magnet.add|daemon.stop|hub.add|hub.del|hub.say|hub.pm|hub.list|hub.list|share.add|share.rename|share.del|share.list|share.refresh|list.download|hub.getchat|search.send|search.getresults|show.version|show.ratio|queue.setpriority|queue.move|queue.remove|queue.listtargets|queue.list|queue.getsources|hash.status|hash.pause|methods.list|queue.matchlists|hub.getusers|hub.getuserinfo|hub.listfulldesc";
+}
+
+void ServerThread::matchAllList() {
+    QueueManager::getInstance()->matchAllListings();
+}
+
+//void ServerThread::getHubUserList(StringMap& userlist, const string& huburl) {
+    //ClientIter i = clientsMap.find(huburl);
+    //if (i != clientsMap.end() && clientsMap[i->first].curclient != NULL) {
+        //userlist = clientsMap[i->first].curuserlist;
+    //}
+//}
+
+void ServerThread::getHubUserList(string& userlist, const string& huburl, const string& separator) {
+    string tmp = separator.empty()? ";" : separator;
+    if (clientsMap[huburl].curclient != NULL) {
+        StringMap& ll = clientsMap[huburl].curuserlist;
+        for (auto it = ll.begin(); it != ll.end(); ++it) {
+            userlist += it->first;
+            userlist += tmp;
+        }
+    }
+}
+
+void ServerThread::getParamsUser(StringMap& params, Identity& id)
+{
+    if(id.supports(AdcHub::ADCS_FEATURE) && id.supports(AdcHub::SEGA_FEATURE) &&
+    ((id.supports(AdcHub::TCP4_FEATURE) && id.supports(AdcHub::UDP4_FEATURE)) || id.supports(AdcHub::NAT0_FEATURE)))
+        params.insert(StringMap::value_type("Icon", "dc++"));
+    else
+        params.insert(StringMap::value_type("Icon", "normal"));
+
+    if (id.getUser()->isSet(User::PASSIVE))
+        params["Icon"] += "-fw";
+
+    if (id.isOp())
+    {
+        params["Icon"] += "-op";
+        params.insert(StringMap::value_type("Nick Order", "o" + id.getNick()));
+    }
+    else
+    {
+        params.insert(StringMap::value_type("Nick Order", "u" + id.getNick()));
+    }
+
+    params.insert(StringMap::value_type("Nick", id.getNick()));
+    params.insert(StringMap::value_type("Shared", Util::toString(id.getBytesShared())));
+    params.insert(StringMap::value_type("Description", id.getDescription()));
+    params.insert(StringMap::value_type("Tag", id.getTag()));
+    params.insert(StringMap::value_type("Connection", id.getConnection()));
+    params.insert(StringMap::value_type("IP", id.getIp()));
+    params.insert(StringMap::value_type("eMail", id.getEmail()));
+    params.insert(StringMap::value_type("CID", id.getUser()->getCID().toBase32()));
+}
+
+void ServerThread::updateUser(const StringMap& params, Client* cl)
+{
+    const string &cid = params.at("CID");
+    const string &Nick = params.at("Nick");
+    StringMap & item = clientsMap[cl->getHubUrl()].curuserlist;
+    auto it = item.begin();
+    while (it != item.end()) {
+        if (it->second == cid)
+            break;
+        else
+            ++it;
+    }
+
+    if (it == item.end()) {
+        item.insert(StringMap::value_type(Nick, cid));
+        if (isDebug) {printf ("HUB: %s == Add user: %s\n", cl->getHubUrl().c_str(), Nick.c_str()); fflush (stdout);}
+    } else if (it->first != Nick) {
+        // User has changed nick, update userMap and remove the old Nick tag
+        item.erase(it->first);
+        item.insert(StringMap::value_type(Nick, cid));
+        if (isDebug) {printf ("HUB: %s == Update user: %s\n", cl->getHubUrl().c_str(), Nick.c_str()); fflush (stdout);}
+    }
+}
+
+void ServerThread::removeUser(const string& cid, Client* cl)
+{
+    StringMap & item = clientsMap[cl->getHubUrl()].curuserlist;
+    auto it = item.begin();
+    while (it != item.end()) {
+        if (it->second == cid)
+            break;
+        else
+            ++it;
+    }
+    if (it == item.end()) {
+        if (isDebug) {printf ("HUB: %s == ERROR: no user with this cid (%s)\n", cl->getHubUrl().c_str(), cid.c_str()); fflush (stdout);}
+    } else {
+        item.erase(it->first);
+        if (isDebug) {printf ("HUB: %s == Remove user: %s\n", cl->getHubUrl().c_str(), (it->first).c_str()); fflush (stdout);}
+    }
+}
+
+bool ServerThread::getUserInfo(StringMap& userinfo, const string& nick, const string& huburl) {
+    ClientIter i = clientsMap.find(huburl);
+    if (i != clientsMap.end() && clientsMap[i->first].curclient != NULL) {
+        auto it = i->second.curuserlist.find(nick);
+        if (it == i->second.curuserlist.end())
+            return false;
+        UserPtr user = ClientManager::getInstance()->findUser(CID(it->second));
+        if (user && user->isOnline()) {
+            Identity id = ClientManager::getInstance()->getOnlineUserIdentity(user);
+            if (!id.isHidden()) {
+                getParamsUser(userinfo, id);
+                return true;
+            }
+        }
+    }
+    return false;
 }
