@@ -34,6 +34,7 @@
 #include <ifaddrs.h>
 #include <net/if.h>
 #endif
+#include "extra/magnet.h"
 
 using namespace std;
 using namespace dcpp;
@@ -231,7 +232,9 @@ void WulforUtil::openURI(const string &uri)
 {
     GError* error = NULL;
     gchar *argv[3];
-
+    if (!SETTING(MIME_HANDLER).empty())
+        argv[0] = (gchar *)(SETTING(MIME_HANDLER)).c_str();
+    else
 #if defined(__APPLE__)
     argv[0] = (gchar *)"open";
 #elif defined(_WIN32)
@@ -270,10 +273,22 @@ string WulforUtil::colorToString(const GdkColor *color)
 
     g_snprintf(strcolor, sizeof(strcolor), "#%04X%04X%04X",
         color->red, color->green, color->blue);
-
+    //printf("WulforUtil::colorToString{GdkColor} %s\n", strcolor);fflush(stdout);
     return strcolor;
 }
 
+#if GTK_CHECK_VERSION (3,0,0)
+string WulforUtil::colorToString(const GdkRGBA *color)
+{
+    gchar strcolor[14];
+
+    g_snprintf(strcolor, sizeof(strcolor), "#%04X%04X%04X",
+        (uint16_t)(color->red*65535.0), (uint16_t)(color->green*65535.0), (uint16_t)(color->blue*65535.0));
+    //printf("WulforUtil::colorToString{GdkRGBA} %s\n",strcolor);fflush(stdout);
+    return strcolor;
+}
+#endif
+
 GdkPixbuf* WulforUtil::scalePixbuf(const GdkPixbuf *pixbuf, const int width, const int height, GdkInterpType type)
 {
     g_return_val_if_fail(pixbuf != NULL, NULL);
@@ -317,24 +332,15 @@ bool WulforUtil::splitMagnet(const string &magnet, string &name, int64_t &size,
     name = _("Unknown");
     size = 0;
     tth = _("Unknown");
-    string tmp;
 
-    if (!isMagnet(magnet.c_str()) || magnet.size() <= magnetSignature.length())
-        return FALSE;
-
-    tmp = magnet.substr(8);
-    StringTokenizer<string> st(tmp, "&");
-    for (StringIter i = st.getTokens().begin(); i != st.getTokens().end(); ++i) {
-        string str;
-        str=*i;
-        if (str.compare(0, 18, "xt=urn:tree:tiger:") == 0)
-            tth=str.substr(3+15);
-        else if (str.compare(0, 3, "xl=") == 0)
-            size = Util::toInt64(str.substr(3));
-        else if (str.compare(0, 3, "dn=") == 0)
-            name = Util::encodeURI(str.substr(3), true);
+    StringMap params;
+    if (magnet::parseUri(magnet,params)) {
+        tth=params["xt"];
+        size = Util::toInt64(params["xl"]);
+        name = params["dn"];
+        return TRUE;
     }
-    return TRUE;
+    return FALSE;
 }
 
 bool WulforUtil::splitMagnet(const string &magnet, string &line)
