@@ -181,7 +181,10 @@ MainWindow::MainWindow():
     gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(getWidget("progressbarFreeSpaceBar")), TRUE);
 #endif
     // About dialog
-    gchar *comments = g_strdup_printf(_("DC++ Client based on the source code of FreeDC++ and LinuxDC++\n\nEiskaltDC++ version: %s (%s)\nDC++ core version: %s"),
+    gchar *comments = g_strdup_printf(_("DC++ Client based on the source code of FreeDC++ and LinuxDC++\n"
+                                        "\n"
+                                        "EiskaltDC++ version: %s (%s)\n"
+                                        "DC++ core version: %s"),
         EISKALTDCPP_VERSION, EISKALTDCPP_VERSION_SFX, DCVERSIONSTRING);
 
     gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(getWidget("aboutDialog")), comments);
@@ -198,71 +201,76 @@ MainWindow::MainWindow():
     }
 
     // set authors
-    string authors =
-        _("Developers\n") +
-        string("  Boris Pek <tehnick-8@mail.ru>\n")+
-        string("  Eugene Petrov <dhamp@ya.ru>\n")+
-        string("\n")+
-        _("Contributors\n")+
-        string("  troll <troll.freedcpp@gmail.com>\n")+
-        string("    http://code.google.com/p/freedcpp/\n")+
-        string("  Mank <mank@jabbim.cz>\n")+
-        string("    http://sourceforge.net/projects/freedcppmv/\n");
-    const gchar *authors_[] = {authors.c_str(), NULL};
-    gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(getWidget("aboutDialog")), authors_);
+    const gchar * const authors[] =
+    {
+        _("Developers"),
+        "  2010-2013 Boris Pek <tehnick-8@mail.ru>",
+        "  2010-2013 Eugene Petrov <dhamp@ya.ru>",
+        "  ",
+        _("Contributors"),
+        "  2001-2012 LinuxDC++ Team http://launchpad.net/linuxdcpp",
+        "  2009-2011 troll http://code.google.com/p/freedcpp/", // <troll.freedcpp@gmail.com>
+        "  2010-2012 Mank http://sourceforge.net/projects/freedcppmv/", // <mank@jabbim.cz>
+        NULL
+    };
+    gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(getWidget("aboutDialog")),
+        (const gchar **)authors);
 
     string translator_credits =
         _("Russian translation\n")+
-        string("  Boris Pek aka Tehnick <tehnick-8@mail.ru>\n")+
+        string("  2010-2013 Boris Pek aka Tehnick <tehnick-8@mail.ru>\n")+
+        string("  2012 Vyacheslav Tretyakov <tret2003@gmail.com>\n")+
         string("\n")+
         _("Belarusian translation\n")+
-        string("  Paval Shalamitski aka Klyok <i.kliok@gmail.com>\n")+
+        string("  2009-2013 Paval Shalamitski aka Klyok <i.kliok@gmail.com>\n")+
         string("\n")+
         _("Hungarian translation\n")+
-        string("  Akos Berki aka sumo <husumo@gmail.com>\n")+
-        string("  Márk Lutring <marcus@elitemail.hu>\n")+
+        string("  2010-2012 Akos Berki aka sumo <husumo@gmail.com>\n")+
+        string("  2011-2013 Márk Lutring <marcus@elitemail.hu>\n")+
         string("\n")+
         _("French translation\n")+
-        string("  Alexandre Wallimann aka Ale <alexandre.wallimann@gmail.com>\n")+
+        string("  2010-2012 Alexandre Wallimann aka Ale <alexandre.wallimann@gmail.com>\n")+
         string("\n")+
         _("Polish translation\n")+
-        string("  Arahael <arahael@gmail.com>\n")+
+        string("  2010-2012 Arahael <arahael@gmail.com>\n")+
         string("\n")+
         _("Ukrainian translation\n")+
-        string("  Dmytro Demenko <dmytro.demenko@gmail.com>\n")+
+        string("  2010 Dmytro Demenko <dmytro.demenko@gmail.com>\n")+
         string("\n")+
         _("Serbian (Latin) translation\n")+
-        string("  Miroslav Petrovic <miroslav031@gmail.com>\n")+
+        string("  2010-2013 Miroslav Petrovic <miroslav031@gmail.com>\n")+
         string("\n")+
         _("Spanish translation\n")+
-        string("  Gustavo Alvarez aka sL1pKn07 <sl1pkn07@gmail.com>\n")+
+        string("  2010-2013 Gustavo Alvarez aka sL1pKn07 <sl1pkn07@gmail.com>\n")+
+        string("  2012 Francisco Blas Izquierdo Riera aka klondike <klondike at klondike.es>\n")+
         string("\n")+
         _("Bulgarian translation\n")+
-        string("  Rusi Dimitrov aka PsyTrip <dimitrov.rusi@gmail.com>\n")+
+        string("  2010-2012 Rusi Dimitrov aka PsyTrip <dimitrov.rusi@gmail.com>\n")+
         string("\n")+
         _("Slovak translation\n")+
-        string("  Martin Durisin <martin.durisin@gmail.com>\n")+
+        string("  2010-2012 Martin Durisin <martin.durisin@gmail.com>\n")+
         string("\n")+
         _("Czech translation\n")+
-        string("  Uhlik <uhlikx@seznam.cz>\n")+
+        string("  2011-2012 Uhlik <uhlikx@seznam.cz>\n")+
         string("\n")+
         _("German translation\n")+
-        string("  Chris Leick <c.leick@vollbio.de>\n")+
-        string("  Konstantinos Georgokitsos <kgeorgokitsos@yahoo.de>\n")+
-        string("  Tillmann Karras <tilkax@gmail.com>\n")+
-        string("  Benjamin Weber <be.w@mail.ru>\n")+
+        string("  2011 Chris Leick <c.leick@vollbio.de>\n")+
+        string("  2011-2012 Konstantinos Georgokitsos <kgeorgokitsos@yahoo.de>\n")+
+        string("  2011-2012 Tillmann Karras <tilkax@gmail.com>\n")+
+        string("  2012-2013 Benjamin Weber <be.w@mail.ru>\n")+
         string("\n")+
         _("Greek translation\n")+
-        string("  Konstantinos Georgokitsos <kgeorgokitsos@yahoo.de>\n")+
+        string("  2011-2012 Konstantinos Georgokitsos <kgeorgokitsos@yahoo.de>\n")+
         string("\n")+
         _("Italian translation\n")+
-        string("  Stefano Simoncelli <netcelli@gmail.com>\n")+
+        string("  2012 Stefano Simoncelli <netcelli@gmail.com>\n")+
+        string("  2012 Lorenzo Keller <lorenzo.keller@gmail.com>\n")+
         string("\n");
     gtk_about_dialog_set_translator_credits(GTK_ABOUT_DIALOG(getWidget("aboutDialog")),
         (const gchar *)translator_credits.c_str());
 
     // This has to be set in code in order to activate the link
-    gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(getWidget("aboutDialog")), "http://eiskaltdc.googlecode.com");
+    gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(getWidget("aboutDialog")), "http://eiskaltdc.googlecode.com/");
     gtk_window_set_transient_for(GTK_WINDOW(getWidget("aboutDialog")), window);
 
     // Set all windows to the default icon
@@ -299,6 +307,7 @@ MainWindow::MainWindow():
     g_signal_connect(getWidget("finishedUploads"), "clicked", G_CALLBACK(onFinishedUploadsClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("openFileListMenuItem"), "activate", G_CALLBACK(onOpenFileListClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("openOwnListMenuItem"), "activate", G_CALLBACK(onOpenOwnListClicked_gui), (gpointer)this);
+    g_signal_connect(getWidget("matchAllListMenuItem"), "activate", G_CALLBACK(onMatchAllList_gui), (gpointer)this);
     g_signal_connect(getWidget("refreshFileListMenuItem"), "activate", G_CALLBACK(onRefreshFileListClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("quickConnectMenuItem"), "activate", G_CALLBACK(onConnectClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("reconnectMenuItem"), "activate", G_CALLBACK(onReconnectClicked_gui), (gpointer)this);
@@ -342,7 +351,7 @@ MainWindow::MainWindow():
     g_signal_connect(getWidget("homeMenuItem"), "activate", G_CALLBACK(onLinkClicked_gui), NULL);
 
     g_object_set_data_full(G_OBJECT(getWidget("sourceMenuItem")), "link",
-        g_strdup("http://github.com/negativ/eiskaltdcpp/"), g_free);
+        g_strdup("http://github.com/eiskaltdcpp/eiskaltdcpp/"), g_free);
     g_signal_connect(getWidget("sourceMenuItem"), "activate", G_CALLBACK(onLinkClicked_gui), NULL);
 
     g_object_set_data_full(G_OBJECT(getWidget("issueMenuItem")), "link",
@@ -355,7 +364,7 @@ MainWindow::MainWindow():
 
     // Now available: ChangeLog.txt, ChangeLog_ru.txt, ChangeLog_uk.txt
     g_object_set_data_full(G_OBJECT(getWidget("changeLogItem")), "link",
-        g_strdup(_("http://github.com/negativ/eiskaltdcpp/raw/master/ChangeLog.txt")), g_free);
+        g_strdup(_("http://github.com/eiskaltdcpp/eiskaltdcpp/raw/master/ChangeLog.txt")), g_free);
     g_signal_connect(getWidget("changeLogItem"), "activate", G_CALLBACK(onLinkClicked_gui), NULL);
 
     onQuit = FALSE;
@@ -389,13 +398,13 @@ MainWindow::MainWindow():
     Emoticons::start();
     Notify::start();
 
-    if (WGETI("main-window-maximized"))
+    if (WGETB("main-window-maximized"))
         gtk_window_maximize(window);
-    if (WGETI("minimize-tray"))
+    if (WGETB("minimize-tray") && WGETB("always-tray"))
         gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(getWidget("statusIconShowInterfaceItem")), FALSE);
-    if (!WGETI("show-free-space-bar"))
+    if (!WGETB("show-free-space-bar"))
         gtk_widget_hide(getWidget("progressbarFreeSpaceBar"));
-    if (!WGETI("show-transfers"))
+    if (!WGETB("show-transfers"))
         gtk_widget_hide(transfers->getContainer());
 
 #ifdef LUA_SCRIPT
@@ -998,7 +1007,7 @@ void MainWindow::showShareBrowser_gui(UserPtr user, string filename, string dir,
 
     if (entry == NULL)
     {
-        entry = new ShareBrowser(user, filename, dir);
+        entry = new ShareBrowser(user, filename, dir, true);
         addBookEntry_gui(entry);
     }
 
@@ -2038,6 +2047,13 @@ void MainWindow::onOpenOwnListClicked_gui(GtkWidget *widget, gpointer data)
     mw->setMainStatus_gui(_("Loading file list"));
 }
 
+void MainWindow::onMatchAllList_gui(GtkWidget *widget, gpointer data)
+{
+    typedef Func0<MainWindow> F0;
+    F0 *func = new F0((MainWindow *)data, &MainWindow::matchAllList_client);
+    WulforManager::get()->dispatchClientFunc(func);
+}
+
 void MainWindow::onRefreshFileListClicked_gui(GtkWidget *widget, gpointer data)
 {
     typedef Func0<MainWindow> F0;
@@ -2282,6 +2298,11 @@ void MainWindow::openOwnList_client(bool useSetting)
     WulforManager::get()->dispatchGuiFunc(func);
 }
 
+void MainWindow::matchAllList_client()
+{
+    QueueManager::getInstance()->matchAllListings();
+}
+
 void MainWindow::on(LogManagerListener::Message, time_t t, const string &message) noexcept
 {
     typedef Func2<MainWindow, string, time_t> F2;
@@ -2369,9 +2390,11 @@ void MainWindow::on(TimerManagerListener::Second, uint64_t ticks) noexcept
         float freepercent = 1.0f*(total-available)/total;
         string freespace = _("Free ") + Util::formatBytes(available);
         //g_print("%s %f\n", format.c_str(), percent);
-        typedef Func2<MainWindow, string, float> F2;
-        F2 *f2 = new F2(this, &MainWindow::updateFreespaceBar_gui, freespace, freepercent);
-        WulforManager::get()->dispatchGuiFunc(f2);
+        if (freepercent > 0 && freepercent <= 1.0) {
+            typedef Func2<MainWindow, string, float> F2;
+            F2 *f2 = new F2(this, &MainWindow::updateFreespaceBar_gui, freespace, freepercent);
+            WulforManager::get()->dispatchGuiFunc(f2);
+        }
 #endif //FREE_SPACE_BAR_C
     }
 
@@ -2534,3 +2557,36 @@ void MainWindow::updateFreespaceBar_gui(string freespace, float freepercent)
     gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(getWidget("progressbarFreeSpaceBar")), freepercent);
 
 }
+
+void MainWindow::parsePartial_gui(HintedUser user, string txt)
+{
+    bool raise = !WGETB("popunder-filelist");
+    BookEntry *entry = findBookEntry(Entry::SHARE_BROWSER, user.user->getCID().toBase32());
+    StringList nicks = ClientManager::getInstance()->getNicks(user);
+    string nick = nicks.empty() ? Util::emptyString : Util::cleanPathChars(nicks[0]) + ".";
+    //string path = QueueManager::getInstance()->getListPath(aUser) + ".xml.bz2";
+    string path = Util::getListPath() + nick + user.user->getCID().toBase32() + ".xml.bz2";
+
+    if(entry != NULL)
+    {
+      dynamic_cast<ShareBrowser*>(entry)->loadXML(txt);
+    }
+    else
+    {
+        if (entry == NULL && !path.empty())
+        {
+            entry = new ShareBrowser(user.user, path, "", false);
+            addBookEntry_gui(entry);
+            dynamic_cast<ShareBrowser*>(entry)->loadXML(txt);
+        }
+    }
+    if (raise)
+        raisePage_gui(entry->getContainer());
+}
+
+/**/
+void MainWindow::on(QueueManagerListener::PartialList, const HintedUser& aUser, const string& text) noexcept {
+    typedef Func2<MainWindow, HintedUser, string> F2;
+    F2 *func = new F2(this,&MainWindow::parsePartial_gui,aUser,text);
+    WulforManager::get()->dispatchGuiFunc(func);
+}
