@@ -40,12 +40,20 @@ Search::Search():
     BookEntry(Entry::SEARCH, _("Search"), "search.ui", generateID()),
     previousGrouping(NOGROUPING)
 {
-#if !GTK_CHECK_VERSION(3,0,0)
-    gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR(getWidget("statusbar1")),FALSE);
+#if GTK_CHECK_VERSION(3,0,0)
+    gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(getWidget("progressbar1")), TRUE);
+#else
     gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR(getWidget("statusbar2")),FALSE);
     gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR(getWidget("statusbar3")),FALSE);
 #endif
 
+    // Initialize variables for search progressbar
+    waitingResults = false;
+    searchStartTime = 0;
+    searchEndTime = 1;
+    setProgress_gui("progressbar1", "", 0.0);
+    gtk_widget_hide(getWidget("statusbox"));
+
     // Initialize the search entries combo box
     if (searchEntriesModel == NULL)
         searchEntriesModel = gtk_combo_box_get_model(GTK_COMBO_BOX(getWidget("comboboxentrySearch")));
@@ -175,6 +183,7 @@ Search::Search():
     g_signal_connect(getWidget("searchByTTHItem"), "activate", G_CALLBACK(onSearchByTTHClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("copyMagnetItem"), "activate", G_CALLBACK(onCopyMagnetClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("getFileListItem"), "activate", G_CALLBACK(onGetFileListClicked_gui), (gpointer)this);
+    g_signal_connect(getWidget("openPartial"), "activate", G_CALLBACK(onPartialFileListOpen_gui), (gpointer)this);
     g_signal_connect(getWidget("matchQueueItem"), "activate", G_CALLBACK(onMatchQueueClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("sendPrivateMessageItem"), "activate", G_CALLBACK(onPrivateMessageClicked_gui), (gpointer)this);
     g_signal_connect(getWidget("addToFavoritesItem"), "activate", G_CALLBACK(onAddFavoriteUserClicked_gui), (gpointer)this);
@@ -194,6 +203,7 @@ Search::~Search()
 {
     ClientManager::getInstance()->removeListener(this);
     SearchManager::getInstance()->removeListener(this);
+    TimerManager::getInstance()->removeListener(this);
 
     gtk_widget_destroy(getWidget("dirChooserDialog"));
     g_object_unref(getWidget("mainMenu"));
@@ -204,6 +214,7 @@ void Search::show()
     initHubs_gui();
     ClientManager::getInstance()->addListener(this);
     SearchManager::getInstance()->addListener(this);
+    TimerManager::getInstance()->addListener(this);
 }
 
 void Search::putValue_gui(const string &str, int64_t size, SearchManager::SizeModes mode, SearchManager::TypeModes type)
@@ -218,7 +229,11 @@ void Search::putValue_gui(const string &str, int64_t size, SearchManager::SizeMo
 
 void Search::initHubs_gui()
 {
+#ifdef DO_NOT_USE_MUTEX
     ClientManager::getInstance()->lock();
+#else // DO_NOT_USE_MUTEX
+    auto lock = ClientManager::getInstance()->lock();
+#endif // DO_NOT_USE_MUTEX
 
     Client::List& clients = ClientManager::getInstance()->getClients();
 
@@ -230,7 +245,9 @@ void Search::initHubs_gui()
             addHub_gui(client->getHubName(), client->getHubUrl());
     }
 
+#ifdef DO_NOT_USE_MUTEX
     ClientManager::getInstance()->unlock();
+#endif // DO_NOT_USE_MUTEX
 }
 
 void Search::addHub_gui(string name, string url)
@@ -424,13 +441,18 @@ void Search::setStatus_gui(string statusBar, string text)
     gtk_statusbar_push(GTK_STATUSBAR(getWidget(statusBar)), 0, text.c_str());
 }
 
+void Search::setProgress_gui(const std::string& progressBar, const std::string& text, float fract)
+{
+    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(getWidget(progressBar)), text.c_str());
+    gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(getWidget(progressBar)), fract);
+}
+
 void Search::search_gui()
 {
     StringList clients;
     GtkTreeIter iter;
 
     string text = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(getWidget("comboboxentrySearch")));
-    //gtk_entry_get_text(GTK_ENTRY(searchEntry));
     if (text.empty())
         return;
 
@@ -537,15 +559,22 @@ void Search::search_gui()
     gtk_list_store_prepend(store, &iter);
     gtk_list_store_set(store, &iter, 0, text.c_str(), -1);
 
+    gtk_widget_show_all(getWidget("statusbox"));
     droppedResult = 0;
     searchHits = 0;
-    setStatus_gui("statusbar1", _("Searching for ") + text + " ...");
-    setStatus_gui("statusbar2", _("0 items"));
-    setStatus_gui("statusbar3", _("0 filtered"));
+    setStatus_gui("statusbar2", _("Found: 0"));
+    setStatus_gui("statusbar3", _("Filtered: 0"));
     setLabel_gui(text);
 
+    searchStartTime = GET_TICK();
+    target = text;
+
     dcdebug(_("Sent ADC extensions : %s\n"), Util::toString(";", exts).c_str());
-    SearchManager::getInstance()->search(clients, text, llsize, (SearchManager::TypeModes)ftype, mode, "manual", exts);
+    uint64_t maxDelayBeforeSearch = SearchManager::getInstance()->search(clients, text, llsize, (SearchManager::TypeModes)ftype, mode, "manual", exts, (void*)this);
+    uint64_t waitingResultsTime = 20000; // just assumption that user receives most of results in 20 seconds
+
+    searchEndTime = searchStartTime + maxDelayBeforeSearch + waitingResultsTime;
+    waitingResults = true;
 
     if (WGETB("clearsearch")) // Only clear if the search was sent.
         gtk_entry_set_text(GTK_ENTRY(searchEntry), "");
@@ -630,7 +659,7 @@ void Search::addResult_gui(const SearchResultPtr result)
         updateParentRow_gui(&parent, &iter);
 
     ++searchHits;
-    setStatus_gui("statusbar2", Util::toString(searchHits) + _(" items"));
+    setStatus_gui("statusbar2", _("Found: ") + Util::toString(searchHits));
 
     if (WGETB("bold-search"))
         setBold_gui();
@@ -1338,7 +1367,7 @@ void Search::onGetFileListClicked_gui(GtkMenuItem *item, gpointer data)
         GtkTreeIter iter;
         GtkTreePath *path;
         GList *list = gtk_tree_selection_get_selected_rows(s->selection, NULL);
-        typedef Func4<Search, string, string, bool, string> F4;
+        typedef Func5<Search, string, string, bool, string, bool> F5;
 
         for (GList *i = list; i; i = i->next)
         {
@@ -1352,7 +1381,41 @@ void Search::onGetFileListClicked_gui(GtkMenuItem *item, gpointer data)
                     string cid = s->resultView.getString(&iter, "CID");
                     string dir = s->resultView.getString(&iter, _("Path"));
                     string hubUrl = s->resultView.getString(&iter, "Hub URL");
-                    F4 *func = new F4(s, &Search::getFileList_client, cid, dir, FALSE, hubUrl);
+                    F5 *func = new F5(s, &Search::getFileList_client, cid, dir, FALSE, hubUrl, TRUE);
+                    WulforManager::get()->dispatchClientFunc(func);
+                }
+                while (parent && WulforUtil::getNextIter_gui(s->sortedFilterModel, &iter, TRUE, FALSE));
+            }
+            gtk_tree_path_free(path);
+        }
+        g_list_free(list);
+    }
+}
+
+void Search::onPartialFileListOpen_gui(GtkMenuItem *item, gpointer data)
+{
+    Search *s = (Search *)data;
+
+    if (gtk_tree_selection_count_selected_rows(s->selection) > 0)
+    {
+        GtkTreeIter iter;
+        GtkTreePath *path;
+        GList *list = gtk_tree_selection_get_selected_rows(s->selection, NULL);
+        typedef Func5<Search, string, string, bool, string, bool> F5;
+
+        for (GList *i = list; i; i = i->next)
+        {
+            path = (GtkTreePath *)i->data;
+            if (gtk_tree_model_get_iter(s->sortedFilterModel, &iter, path))
+            {
+                bool parent = gtk_tree_model_iter_has_child(s->sortedFilterModel, &iter);
+
+                do
+                {
+                    string cid = s->resultView.getString(&iter, "CID");
+                    string dir = s->resultView.getString(&iter, _("Path"));
+                    string hubUrl = s->resultView.getString(&iter, "Hub URL");
+                    F5 *func = new F5(s, &Search::getFileList_client, cid, dir, FALSE, hubUrl, FALSE);
                     WulforManager::get()->dispatchClientFunc(func);
                 }
                 while (parent && WulforUtil::getNextIter_gui(s->sortedFilterModel, &iter, TRUE, FALSE));
@@ -1372,7 +1435,7 @@ void Search::onMatchQueueClicked_gui(GtkMenuItem *item, gpointer data)
         GtkTreeIter iter;
         GtkTreePath *path;
         GList *list = gtk_tree_selection_get_selected_rows(s->selection, NULL);
-        typedef Func4<Search, string, string, bool, string> F4;
+        typedef Func5<Search, string, string, bool, string, bool> F5;
 
         for (GList *i = list; i; i = i->next)
         {
@@ -1385,7 +1448,7 @@ void Search::onMatchQueueClicked_gui(GtkMenuItem *item, gpointer data)
                 {
                     string cid = s->resultView.getString(&iter, "CID");
                     string hubUrl = s->resultView.getString(&iter, "Hub URL");
-                    F4 *func = new F4(s, &Search::getFileList_client, cid, "", TRUE, hubUrl);
+                    F5 *func = new F5(s, &Search::getFileList_client, cid, "", TRUE, hubUrl, TRUE);
                     WulforManager::get()->dispatchClientFunc(func);
                 }
                 while (parent && WulforUtil::getNextIter_gui(s->sortedFilterModel, &iter, TRUE, FALSE));
@@ -1741,7 +1804,7 @@ void Search::addSource_client(string source, string cid, int64_t size, string tt
     }
 }
 
-void Search::getFileList_client(string cid, string dir, bool match, string hubUrl)
+void Search::getFileList_client(string cid, string dir, bool match, string hubUrl, bool full)
 {
     if (!cid.empty())
     {
@@ -1755,7 +1818,7 @@ void Search::getFileList_client(string cid, string dir, bool match, string hubUr
                     flags = QueueItem::FLAG_MATCH_QUEUE;
                 else
                     flags = QueueItem::FLAG_CLIENT_VIEW;
-                QueueManager::getInstance()->addList(HintedUser(user, hubUrl), flags, dir);
+                QueueManager::getInstance()->addList(HintedUser(user, hubUrl), full ? flags : QueueItem::FLAG_CLIENT_VIEW | QueueItem::FLAG_PARTIAL_LIST, dir);
             }
         }
         catch (const Exception&)
@@ -1826,6 +1889,18 @@ void Search::on(ClientManagerListener::ClientDisconnected, Client *client) noexc
     }
 }
 
+void Search::on(TimerManagerListener::Second, uint64_t aTick) noexcept {
+    if (waitingResults) {
+        float fraction  = 1.0f*(aTick - searchStartTime)/ (searchEndTime - searchStartTime);
+        if (fraction >= 1.0) {
+            fraction = 1.0;
+            waitingResults = false;
+        }
+        setProgress_gui("progressbar1", _("Searching for ") + target + " ...", fraction);
+    } else
+        setProgress_gui("progressbar1", "", 0.0);
+}
+
 void Search::on(SearchManagerListener::SR, const SearchResultPtr& result) noexcept
 {
     if (searchlist.empty() || result == NULL)
@@ -1838,7 +1913,7 @@ void Search::on(SearchManagerListener::SR, const SearchResultPtr& result) noexce
         if (result->getType() != SearchResult::TYPE_FILE || TTHValue(searchlist[0]) != result->getTTH())
         {
             ++droppedResult;
-            F2 *func = new F2(this, &Search::setStatus_gui, "statusbar3", Util::toString(droppedResult) + _(" filtered"));
+            F2 *func = new F2(this, &Search::setStatus_gui, "statusbar3", _("Filtered: ") + Util::toString(droppedResult));
             WulforManager::get()->dispatchGuiFunc(func);
             return;
         }
@@ -1851,7 +1926,7 @@ void Search::on(SearchManagerListener::SR, const SearchResultPtr& result) noexce
                 (*i->begin() == '-' && i->size() != 1 && Util::findSubString(result->getFile(), i->substr(1)) != (string::size_type)-1))
             {
                 ++droppedResult;
-                F2 *func = new F2(this, &Search::setStatus_gui, "statusbar3", Util::toString(droppedResult) + _(" dropped"));
+                F2 *func = new F2(this, &Search::setStatus_gui, "statusbar3", _("Dropped: ") + Util::toString(droppedResult));
                 WulforManager::get()->dispatchGuiFunc(func);
                 return;
             }
