@@ -146,7 +146,7 @@ WulforSettingsManager::WulforSettingsManager():
     defaultInt.insert(IntMap::value_type("popunder-filelist", 0));
     defaultInt.insert(IntMap::value_type("use-oen-monofont", 0));
     defaultInt.insert(IntMap::value_type("urlhandler", 0));
-    defaultInt.insert(IntMap::value_type("clearsearch", 1));
+    defaultInt.insert(IntMap::value_type("clearsearch", 0));
     defaultInt.insert(IntMap::value_type("use-system-icons", 1));
     defaultInt.insert(IntMap::value_type("sort-favusers-first", 0));
     defaultInt.insert(IntMap::value_type("status-in-chat", 1));
@@ -158,7 +158,7 @@ WulforSettingsManager::WulforSettingsManager():
     defaultInt.insert(IntMap::value_type("show-free-space-bar", 1));
     defaultInt.insert(IntMap::value_type("show-transfers", 1));
     defaultInt.insert(IntMap::value_type("last-search-type", 0));
-    
+    defaultInt.insert(IntMap::value_type("hub-nickview-left", 0));
 
     defaultString.insert(StringMap::value_type("magnet-choose-dir", SETTING(DOWNLOAD_DIRECTORY)));
     defaultString.insert(StringMap::value_type("downloadqueue-order", ""));
@@ -502,7 +502,7 @@ bool WulforSettingsManager::getPreviewApp(string &name, PreviewApp::size &index)
     return false;
 }
 
-const std::string WulforSettingsManager::parseCmd(const std::string cmd)
+const std::string WulforSettingsManager::parseCmd(const std::string &cmd)
 {
     StringTokenizer<string> sl(cmd, ' ');
         if (sl.getTokens().size() == 2) {
