@@ -178,7 +178,11 @@ private:
     void loadList(){
         VarMap params;
 
+#ifdef DO_NOT_USE_MUTEX
         FinishedManager::getInstance()->lockLists();
+#else // DO_NOT_USE_MUTEX
+        auto lock = FinishedManager::getInstance()->lockLists();
+#endif // DO_NOT_USE_MUTEX
         const FinishedManager::MapByFile &list = FinishedManager::getInstance()->getMapByFile(isUpload);
         const FinishedManager::MapByUser &user = FinishedManager::getInstance()->getMapByUser(isUpload);
 
@@ -198,7 +202,9 @@ private:
             model->addUser(params);;
         }
 
+#ifdef DO_NOT_USE_MUTEX
         FinishedManager::getInstance()->unlockLists();
+#endif // DO_NOT_USE_MUTEX
 
         AsyncRunner *runner = new AsyncRunner(this);
 
