@@ -91,7 +91,7 @@ using namespace std;
 class MainWindowPrivate {
 public:
         typedef QList<QAction*> ActionList;
-        
+
         bool isUnload;
         bool exitBegin;
 
@@ -126,6 +126,7 @@ public:
         QAction *fileFileListBrowser;
         QAction *fileFileHasher;
         QAction *fileFileListBrowserLocal;
+        QAction *fileFileListMatchAll;
         QAction *fileRefreshShareHashProgress;
         QAction *fileOpenLogFile;
         QAction *fileOpenDownloadDirectory;
@@ -207,6 +208,8 @@ public:
         ActionList fileMenuActions;
         ActionList hubsMenuActions;
         ActionList toolsMenuActions;
+
+        QMenu *favHubMenu;
 };
 
 static const QString &TOOLBUTTON_STYLE = "mainwindow/toolbar-toolbutton-style";
@@ -218,7 +221,7 @@ MainWindow::MainWindow (QWidget *parent):
         d_ptr(new MainWindowPrivate())
 {
     Q_D(MainWindow);
-    
+
     d->statusLabel = NULL;
     d->fBar = NULL;
     d->sBar = NULL;
@@ -228,7 +231,8 @@ MainWindow::MainWindow (QWidget *parent):
 #ifdef USE_JS
     d->scriptConsole = NULL;
 #endif
-    
+    d->favHubMenu = NULL;
+
     d->exitBegin = false;
 
     if (WBGET(WB_ANTISPAM_ENABLED)){
@@ -289,10 +293,10 @@ MainWindow::MainWindow (QWidget *parent):
 
 HashProgress* MainWindow::progress_dialog() {
     Q_D(MainWindow);
-    
+
     if( d->_progress_dialog == NULL )
         d->_progress_dialog = new HashProgress(this);
-    
+
     return d->_progress_dialog;
 }
 
@@ -306,28 +310,28 @@ MainWindow::~MainWindow(){
         AntiSpam::getInstance()->saveSettings();
         AntiSpam::deleteInstance();
     }
-    
+
     Q_D(MainWindow);
-    
+
     delete d->arena;
     delete d->fBar;
     delete d->sBar;
 
     ShortcutManager::deleteInstance();
     SearchBlacklist::deleteInstance();
-    
+
     delete d_ptr;
 }
 
 void MainWindow::setUnload ( bool b ) {
-    Q_D(MainWindow); 
-    
+    Q_D(MainWindow);
+
     d->isUnload = b;
 }
 
 void MainWindow::closeEvent(QCloseEvent *c_e){
     Q_D(MainWindow);
-    
+
     if (!d->isUnload && WBGET(WB_TRAY_ENABLED)){
         hide();
         c_e->ignore();
@@ -388,15 +392,15 @@ void MainWindow::closeEvent(QCloseEvent *c_e){
 
 void MainWindow::beginExit(){
     Q_D(MainWindow);
-    
+
     d->exitBegin = true;
-    
+
     setUnload(true);
 }
 
 void MainWindow::show(){
     Q_D(MainWindow);
-    
+
     if (d->showMax)
         showMaximized();
     else
@@ -405,7 +409,7 @@ void MainWindow::show(){
 
 void MainWindow::showEvent(QShowEvent *e){
     Q_D(MainWindow);
-    
+
     if (!d->showMax && d->w > 0 && d->h > 0 && d->w != width() && d->h != height())
         this->resize(QSize(d->w, d->h));
 
@@ -460,7 +464,7 @@ void MainWindow::showEvent(QShowEvent *e){
 
 void MainWindow::hideEvent(QHideEvent *e){
     Q_D(MainWindow);
-    
+
     d->showMax = isMaximized();
 
     if (!d->showMax){
@@ -484,7 +488,7 @@ void MainWindow::hideEvent(QHideEvent *e){
 
 bool MainWindow::eventFilter(QObject *obj, QEvent *e){
     Q_D(MainWindow);
-    
+
     if (e->type() == QEvent::WindowActivate) {
         redrawToolPanel();
     }
@@ -504,7 +508,7 @@ bool MainWindow::eventFilter(QObject *obj, QEvent *e){
 
 void MainWindow::init(){
     Q_D(MainWindow);
-    
+
     setObjectName("MainWindow");
 
     connect(this, SIGNAL(coreLogMessage(QString)), this, SLOT(setStatusMessage(QString)), Qt::QueuedConnection);
@@ -556,7 +560,7 @@ void MainWindow::init(){
     loadSettings();
 
     connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(slotExit()));
-    
+
     connect(ArenaWidgetManager::getInstance(), SIGNAL(activated(ArenaWidget*)), this, SLOT(mapWidgetOnArena(ArenaWidget*)));
     connect(ArenaWidgetManager::getInstance(), SIGNAL(added(ArenaWidget*)),     this, SLOT(insertWidget(ArenaWidget*)));
     connect(ArenaWidgetManager::getInstance(), SIGNAL(removed(ArenaWidget*)),   this, SLOT(removeWidget(ArenaWidget*)));
@@ -685,6 +689,11 @@ void MainWindow::initActions(){
         d->fileFileListBrowser->setIcon(WU->getPixmap(WulforUtil::eiOPENLIST));
         connect(d->fileFileListBrowser, SIGNAL(triggered()), this, SLOT(slotFileBrowseFilelist()));
 
+        d->fileFileListMatchAll = new QAction("", this);
+        d->fileFileListMatchAll->setObjectName("fileFileListMatchAll");
+        //d->fileFileListMatchAll->setIcon(WU->getPixmap(WulforUtil::eiOPENLIST));
+        connect(d->fileFileListMatchAll, SIGNAL(triggered()), this, SLOT(slotFileMatchAllList()));
+
         d->fileFileHasher = new QAction("", this);
         d->fileFileHasher->setObjectName("fileFileHasher");
         d->fileFileHasher->setIcon(WU->getPixmap(WulforUtil::eiOPENLIST));
@@ -941,6 +950,7 @@ void MainWindow::initActions(){
                 << separator3
                 << d->fileFileListBrowser
                 << d->fileFileListBrowserLocal
+                << d->fileFileListMatchAll
                 << d->fileRefreshShareHashProgress
                 << separator0
                 << d->fileOpenLogFile
@@ -1120,7 +1130,7 @@ void MainWindow::initMenuBar(){
     menuBar()->setParent(NULL);
     connect(this, SIGNAL(destroyed()), menuBar(), SLOT(deleteLater()));
 #endif
-    
+
     Q_D(MainWindow);
 
     {
@@ -1180,7 +1190,7 @@ void MainWindow::initMenuBar(){
 
 void MainWindow::initStatusBar(){
     Q_D(MainWindow);
-    
+
     d->statusLabel = new QLabel(statusBar());
     d->statusLabel->setFrameShadow(QFrame::Sunken);
     d->statusLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
@@ -1244,7 +1254,7 @@ void MainWindow::initStatusBar(){
 
 void MainWindow::initSearchBar(){
     Q_D(MainWindow);
-    
+
     d->searchLineEdit = new LineEdit(this);
 
     connect(d->searchLineEdit,   SIGNAL(returnPressed()), this, SLOT(slotToolsSearch()));
@@ -1252,7 +1262,7 @@ void MainWindow::initSearchBar(){
 
 void MainWindow::retranslateUi(){
     Q_D(MainWindow);
-    
+
     //Retranslate menu actions
     {
         d->menuFile->setTitle(tr("&File"));
@@ -1269,6 +1279,8 @@ void MainWindow::retranslateUi(){
 
         d->fileFileListBrowserLocal->setText(tr("Open own filelist"));
 
+        d->fileFileListMatchAll->setText(tr("Match all listings"));
+
         d->fileRefreshShareHashProgress->setText(tr("Refresh share"));
 
         d->fileHideWindow->setText(tr("Hide window"));
@@ -1394,7 +1406,7 @@ void MainWindow::retranslateUi(){
 
 void MainWindow::initToolbar(){
     Q_D(MainWindow);
-    
+
     d->fBar = new ToolBar(this);
     d->fBar->setObjectName("fBar");
 
@@ -1411,6 +1423,8 @@ void MainWindow::initToolbar(){
         }
     }
 
+    initFavHubMenu();
+
     d->fBar->setContextMenuPolicy(Qt::CustomContextMenu);
     d->fBar->setMovable(true);
     d->fBar->setFloatable(true);
@@ -1427,10 +1441,10 @@ void MainWindow::initToolbar(){
         MultiLineToolBar *mBar = new MultiLineToolBar(this);
         mBar->setContextMenuPolicy(Qt::CustomContextMenu);
         mBar->setVisible(WBGET(WB_WIDGETS_PANEL_VISIBLE));
-        
+
         connect(d->nextTabShortCut, SIGNAL(triggered()), mBar, SIGNAL(nextTab()));
         connect(d->prevTabShortCut, SIGNAL(triggered()), mBar, SIGNAL(prevTab()));
-        
+
         addToolBar(mBar);
     }
     else if (!WBGET(WB_MAINWINDOW_USE_SIDEBAR) && !WBGET(WB_MAINWINDOW_USE_M_TABBAR)){
@@ -1444,7 +1458,7 @@ void MainWindow::initToolbar(){
         tBar->setContextMenuPolicy(Qt::CustomContextMenu);
 
         addToolBar(tBar);
-        
+
         connect(d->nextTabShortCut, SIGNAL(triggered()), tBar, SLOT(nextTab()));
         connect(d->prevTabShortCut, SIGNAL(triggered()), tBar, SLOT(prevTab()));
     }
@@ -1456,14 +1470,14 @@ void MainWindow::initToolbar(){
     d->sBar->setMovable(true);
     d->sBar->setFloatable(true);
     d->sBar->setAllowedAreas(Qt::AllToolBarAreas);
-    
+
     addToolBar(d->sBar);
 }
 
 void MainWindow::initSideBar(){
     if (!WBGET(WB_MAINWINDOW_USE_SIDEBAR))
         return;
-    
+
     Q_D(MainWindow);
 
     d->sideDock = new QDockWidget("", this);
@@ -1479,9 +1493,29 @@ void MainWindow::initSideBar(){
     connect(d->sideDock, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotSideBarDockMenu()));
 }
 
+void MainWindow::initFavHubMenu() {
+    Q_D(MainWindow);
+
+    if (!d->fBar)
+        return;
+
+    if (d->favHubMenu == NULL) {
+        d->favHubMenu = new QMenu(this);
+
+        connect(d->favHubMenu, SIGNAL(aboutToShow()), this, SLOT(slotUpdateFavHubMenu()));
+        connect(d->favHubMenu, SIGNAL(triggered(QAction*)), this, SLOT(slotConnectFavHub(QAction*)));
+    }
+
+    QToolButton * btn = qobject_cast<QToolButton *>(d->fBar->widgetForAction(d->hubsFavoriteHubs));
+    if (btn) {
+        btn->setMenu(d->favHubMenu);
+        btn->setPopupMode(QToolButton::MenuButtonPopup);
+    }
+}
+
 QObject *MainWindow::getToolBar(){
     Q_D(MainWindow);
-    
+
     if (!d->fBar)
         return NULL;
 
@@ -1576,7 +1610,7 @@ void MainWindow::newHubFrame(QString address, QString enc){
 
         return;
     }
-    
+
     fr = ArenaWidgetFactory().create<HubFrame, QWidget*, QString, QString>(this, address, enc);
 
     ArenaWidgetManager::getInstance()->activate(fr);
@@ -1584,7 +1618,7 @@ void MainWindow::newHubFrame(QString address, QString enc){
 
 void MainWindow::updateStatus(const QMap<QString, QString> &map){
     Q_D(MainWindow);
-     
+
     if (!d->statusLabel)
         return;
 
@@ -1719,7 +1753,7 @@ void MainWindow::updateHashProgressStatus() {
 
 void MainWindow::setStatusMessage(QString msg){
     Q_D(MainWindow);
-    
+
     QFontMetrics m(d->msgLabel->font());
     QString pure_msg = msg;
 
@@ -1761,9 +1795,7 @@ void MainWindow::autoconnect(){
     }
 }
 
-void MainWindow::parseCmdLine(){
-    QStringList args = qApp->arguments();
-
+void MainWindow::parseCmdLine(const QStringList &args){
     foreach (const QString &arg, args){
         if (arg.startsWith("magnet:?")){
             Magnet m(this);
@@ -1790,22 +1822,7 @@ void MainWindow::parseInstanceLine(QString data){
     }
 
     QStringList args = data.split("\n", QString::SkipEmptyParts);
-
-    foreach (const QString &arg, args){
-        if (arg.startsWith("magnet:?xt=urn:tree:tiger:")){
-            Magnet m(this);
-            m.setLink(arg);
-            if (WIGET(WI_DEF_MAGNET_ACTION) == 0) {
-                m.exec();
-            }
-        }
-        else if (arg.startsWith("dchub://")){
-            newHubFrame(arg, "");
-        }
-        else if (arg.startsWith("adc://") || arg.startsWith("adcs://")){
-            newHubFrame(arg, "UTF-8");
-        }
-    }
+    parseCmdLine(args);
 }
 
 void MainWindow::browseOwnFiles(){
@@ -1832,6 +1849,10 @@ void MainWindow::slotFileBrowseFilelist(){
     }
 }
 
+void MainWindow::slotFileMatchAllList(){
+    QueueManager::getInstance()->matchAllListings();
+}
+
 void MainWindow::redrawToolPanel(){
     Q_D(MainWindow);
 
@@ -1857,19 +1878,19 @@ void MainWindow::redrawToolPanel(){
 
     if (!has_unread)
         Notify->resetTrayIcon();
-    
+
     emit redrawWidgetPanels();
 }
 
 void MainWindow::mapWidgetOnArena(ArenaWidget *awgt){
     Q_D(MainWindow);
-    
+
     if (!(awgt && awgt->getWidget())){
         d->arena->setWidget(NULL);
-        
+
         return;
     }
-    
+
     if (d->arena->widget() != awgt->getWidget())
         d->arena->setWidget(awgt->getWidget());
 
@@ -1896,33 +1917,33 @@ void MainWindow::mapWidgetOnArena(ArenaWidget *awgt){
 void MainWindow::insertWidget ( ArenaWidget* awgt ) {
     if (!awgt || (awgt && (awgt->state() & ArenaWidget::Hidden)))
         return;
-    
+
     Q_D(MainWindow);
-    
+
     QAction *act = d->menuWidgets->addAction(awgt->getPixmap(), awgt->getArenaShortTitle());
-    
+
     d->menuWidgetsHash.insert(act, awgt);
-    
+
     connect(act, SIGNAL(triggered(bool)), this, SLOT(slotWidgetsToggle()));
 }
 
 void MainWindow::removeWidget ( ArenaWidget* awgt ) {
     Q_D(MainWindow);
-    
+
     QAction *act = d->menuWidgetsHash.key(awgt);
-    
+
     if (!act)
         return;
-    
+
     d->menuWidgetsHash.remove(act);
-    
+
     act->deleteLater();
 }
 
 void MainWindow::updated ( ArenaWidget* awgt ) {
     if (!awgt)
         return;
-    
+
     if (awgt->state() & ArenaWidget::Hidden)
         removeWidget(awgt);
     else
@@ -1931,7 +1952,7 @@ void MainWindow::updated ( ArenaWidget* awgt ) {
 
 void MainWindow::addActionOnToolBar(QAction *new_act){
     Q_D(MainWindow);
-    
+
     if (!d->fBar || d->toolBarActions.contains(new_act))
         return;
 
@@ -1941,7 +1962,7 @@ void MainWindow::addActionOnToolBar(QAction *new_act){
 
 void MainWindow::remActionFromToolBar(QAction *act){
     Q_D(MainWindow);
-    
+
     if (!d->fBar || !d->toolBarActions.contains(act))
         return;
 
@@ -1952,7 +1973,7 @@ void MainWindow::remActionFromToolBar(QAction *act){
 void MainWindow::toggleSingletonWidget(ArenaWidget *a){
     if (!a)
         throw std::runtime_error(_tq(Q_FUNC_INFO) + ": NULL argument");
-    
+
     if (sender() && qobject_cast<QAction*>(sender()) && a->getWidget()){
         QAction *act = reinterpret_cast<QAction*>(sender());;
 
@@ -1960,7 +1981,7 @@ void MainWindow::toggleSingletonWidget(ArenaWidget *a){
 
         a->setToolButton(act);
     }
-   
+
     if (!a->getWidget()->isVisible())
         ArenaWidgetManager::getInstance()->activate(a);
     else
@@ -1971,7 +1992,7 @@ void MainWindow::toggleMainMenu(bool showMenu){
     static QAction *compactMenus = NULL;
 
     menuBar()->setVisible(showMenu);
-    
+
     Q_D(MainWindow);
 
     if (showMenu){
@@ -2141,7 +2162,7 @@ void MainWindow::slotToolsSearch() {
     QLineEdit *le = qobject_cast<QLineEdit *> ( sender() );
 
     Q_D(MainWindow);
-    
+
     if ( le != d->searchLineEdit )
         return;
 
@@ -2189,17 +2210,17 @@ void MainWindow::slotToolsAntiSpam(){
     AntiSpamFrame fr(this);
 
     fr.exec();
-    
+
     Q_D(MainWindow);
 
     d->toolsAntiSpam->setChecked(AntiSpam::getInstance() != NULL);
 }
 
-void MainWindow::slotToolsIPFilter(){   
+void MainWindow::slotToolsIPFilter(){
     IPFilterFrame fr(this);
 
     fr.exec();
-    
+
     Q_D(MainWindow);
 
     d->toolsIPFilter->setChecked(IPFilter::getInstance() != NULL);
@@ -2207,7 +2228,7 @@ void MainWindow::slotToolsIPFilter(){
 
 void MainWindow::slotToolsAutoAway(){
     Q_D(MainWindow);
-    
+
     WBSET(WB_APP_AUTO_AWAY, d->toolsAutoAway->isChecked());
 }
 
@@ -2278,11 +2299,11 @@ void MainWindow::slotJSFileChanged(const QString &script){
 
 void MainWindow::slotToolsJSConsole(){
     Q_D(MainWindow);
-    
+
 #ifdef USE_JS
     if (!d->scriptConsole)
         d->scriptConsole = new ScriptConsole(this);
-    
+
     d->scriptConsole->setWindowModality(Qt::NonModal);
     d->scriptConsole->show();
     d->scriptConsole->raise();
@@ -2314,7 +2335,7 @@ void MainWindow::slotToolsSettings(){
     s.exec();
 
     reloadSomeSettings();
-    
+
     Q_D(MainWindow);
 
     //reload some settings
@@ -2326,7 +2347,7 @@ void MainWindow::slotToolsSettings(){
 
 void MainWindow::slotToolsTransfer(bool toggled){
     Q_D(MainWindow);
-    
+
     if (toggled){
         d->transfer_dock->setVisible(true);
         d->transfer_dock->setWidget(TransferView::getInstance());
@@ -2350,7 +2371,7 @@ void MainWindow::slotPanelMenuActionClicked(){
 
     if (act == 0)
         return;
-    
+
     Q_D(MainWindow);
 
     if (act == d->panelsWidgets){
@@ -2362,7 +2383,7 @@ void MainWindow::slotPanelMenuActionClicked(){
         }
         else if (d->sideDock)
             d->sideDock->setVisible(d->panelsWidgets->isChecked());
-        
+
         WBSET(WB_WIDGETS_PANEL_VISIBLE, d->panelsWidgets->isChecked());
     }
     else if (act == d->panelsTools){
@@ -2377,7 +2398,7 @@ void MainWindow::slotPanelMenuActionClicked(){
 
 void MainWindow::slotChatClear(){
     Q_D(MainWindow);
-    
+
     HubFrame *fr = qobject_cast<HubFrame *>(d->arena->widget());
     PMWindow *pm = qobject_cast<PMWindow *>(d->arena->widget());
 
@@ -2389,7 +2410,7 @@ void MainWindow::slotChatClear(){
 
 void MainWindow::slotFind(){
     Q_D(MainWindow);
-    
+
     if (!d->arena->widget() || !qobject_cast<ArenaWidget*>(d->arena->widget()))
         return;
 
@@ -2406,7 +2427,7 @@ void MainWindow::slotChatDisable(){
 
 void MainWindow::slotWidgetsToggle(){
     Q_D(MainWindow);
-    
+
     QAction *act = reinterpret_cast<QAction*>(sender());
     auto it = d->menuWidgetsHash.find(act);
 
@@ -2437,7 +2458,7 @@ void MainWindow::slotShowMainMenu() {
 
 void MainWindow::slotHideWindow(){
     Q_D(MainWindow);
-    
+
     if (!d->isUnload && isActiveWindow() && WBGET(WB_TRAY_ENABLED)) {
         hide();
     }
@@ -2445,7 +2466,7 @@ void MainWindow::slotHideWindow(){
 
 void MainWindow::slotHideProgressSpace() {
     Q_D(MainWindow);
-    
+
     if (WBGET(WB_SHOW_FREE_SPACE)) {
         d->progressSpace->hide();
         d->toolsHideProgressSpace->setText(tr("Show free space bar"));
@@ -2461,7 +2482,7 @@ void MainWindow::slotHideProgressSpace() {
 
 void MainWindow::slotHideLastStatus(){
     Q_D(MainWindow);
-    
+
     bool st = WBGET(WB_LAST_STATUS);
 
     st = !st;
@@ -2478,7 +2499,7 @@ void MainWindow::slotHideLastStatus(){
 
 void MainWindow::slotHideUsersStatistics(){
     Q_D(MainWindow);
-    
+
     bool st = WBGET(WB_USERS_STATISTICS);
 
     st = !st;
@@ -2501,7 +2522,7 @@ void MainWindow::slotExit(){
 
 void MainWindow::slotToolbarCustomization() {
     Q_D(MainWindow);
-    
+
     QMenu *m = new QMenu(this);
 
     QMenu *toolButtonStyle = new QMenu(tr("Button style"), this);
@@ -2539,9 +2560,9 @@ void MainWindow::slotToolbarCustomization() {
 
 void MainWindow::slotToolbarCustomizerDone(const QList<QAction*> &enabled){
     Q_D(MainWindow);
-    
+
     d->fBar->clear();
-    
+
     QStringList enabled_list;
 
     foreach (QAction *act, enabled){
@@ -2552,18 +2573,20 @@ void MainWindow::slotToolbarCustomizerDone(const QList<QAction*> &enabled){
         enabled_list.push_back(act->objectName());
     }
 
+    initFavHubMenu();
+
     WSSET(WS_MAINWINDOW_TOOLBAR_ACTS, enabled_list.join(";").toAscii().toBase64());
 }
 
 void MainWindow::slotAboutOpenUrl(){
     Q_D(MainWindow);
-    
+
     QAction *act = qobject_cast<QAction *>(sender());
     if (act == d->aboutHomepage){
         QDesktopServices::openUrl(QUrl("http://code.google.com/p/eiskaltdc/"));
     }
     else if (act == d->aboutSource){
-        QDesktopServices::openUrl(QUrl("http://github.com/negativ/eiskaltdcpp/"));
+        QDesktopServices::openUrl(QUrl("http://github.com/eiskaltdcpp/eiskaltdcpp/"));
     }
     else if (act == d->aboutIssues){
         QDesktopServices::openUrl(QUrl("http://code.google.com/p/eiskaltdc/issues/list"));
@@ -2573,7 +2596,7 @@ void MainWindow::slotAboutOpenUrl(){
     }
     else if (act == d->aboutChangelog){
         // Now available: ChangeLog.txt, ChangeLog_ru.txt, ChangeLog_uk.txt
-        QDesktopServices::openUrl(QUrl(tr("http://github.com/negativ/eiskaltdcpp/raw/master/ChangeLog.txt")));
+        QDesktopServices::openUrl(QUrl(tr("http://github.com/eiskaltdcpp/eiskaltdcpp/raw/master/ChangeLog.txt")));
     }
 }
 
@@ -2594,13 +2617,15 @@ void MainWindow::slotAboutClient(){
                      .arg(EISKALTDCPP_VERSION)
                      .arg(EISKALTDCPP_VERSION_SFX));
 
-    a.label_ABOUT->setTextFormat(Qt::RichText);
+    QString html_format = "a { text-decoration:none; }\n"
+                          "a:hover { text-decoration: underline; }\n";
 
     QString about_text = tr("EiskaltDC++ is a graphical client for Direct Connect and ADC protocols.<br/><br/>"
                             ""
                             "DC++ core version: %1 (modified)<br/><br/>"
                             ""
-                            "Home page: <a href=\"http://code.google.com/p/eiskaltdc/\">http://code.google.com/p/eiskaltdc/</a><br/><br/>"
+                            "Home page: <a href=\"http://code.google.com/p/eiskaltdc/\">"
+                            "http://code.google.com/p/eiskaltdc/</a><br/><br/>"
                             ""
                             "Total up: <b>%2</b><br/>"
                             "Total down: <b>%3</b><br/>"
@@ -2612,112 +2637,109 @@ void MainWindow::slotAboutClient(){
 
     a.label_ABOUT->setText(about_text);
 
+    a.textBrowser_AUTHORS->document()->setDefaultStyleSheet(html_format);
+
     a.textBrowser_AUTHORS->setText(
         tr("Please use <a href=\"http://code.google.com/p/eiskaltdc/issues/list\">"
         "http://code.google.com/p/eiskaltdc/issues/list</a> to report bugs.<br/>")+
         QString("<br/>")+
         tr("<b>Developers</b><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Andrey Karlov &lt;dein.negativ@gmail.com&gt;<br/>")+
-        tr("&nbsp; (main developer 0.4.10 and later)<br/>")+
+        QString("&nbsp; 2009-2012 <a href=\"mailto:dein.negativ@gmail.com\">Andrey Karlov</a><br/>")+
+        tr("&nbsp;&nbsp;&nbsp; (main developer since version 0.4.10)<br/>")+
         QString("<br/>")+
-        QString("&nbsp; Boris Pek aka Tehnick &lt;tehnick-8@mail.ru&gt;<br/>")+
-        tr("&nbsp; (maintainer and developer 1.89.0 and later)<br/>")+
+        QString("&nbsp; 2009-2013 <a href=\"mailto:dhamp@ya.ru\">Eugene Petrov</a><br/>")+
+        tr("&nbsp;&nbsp;&nbsp; (Arch Linux maintainer and developer since version 0.4.10)<br/>")+
         QString("<br/>")+
-        QString("&nbsp; Eugene Petrov &lt;dhamp@ya.ru&gt;<br/>")+
-        tr("&nbsp; (maintainer and developer 0.4.10 and later)<br/>")+
+        QString("&nbsp; 2010-2013 <a href=\"mailto:tehnick-8@mail.ru\">Boris Pek</a> aka Tehnick<br/>")+
+        tr("&nbsp;&nbsp;&nbsp; (Debian/Ubuntu maintainer and developer since version 1.89.0)<br/>")+
         QString("<br/>")+
-        QString("&nbsp; Alexandr Tkachev &lt;tka4ev@gmail.com&gt;<br/>")+
-        tr("&nbsp; (developer 2.0.3 and later)<br/>")+
+        QString("&nbsp; 2010-2013 <a href=\"mailto:pavelvat@gmail.com\">Pavel Vatagin</a><br/>")+
+        tr("&nbsp;&nbsp;&nbsp; (MS Windows maintainer and developer since version 2.2.4)<br/>")+
         QString("<br/>")+
-        tr("<b>Logo</b><br/>")+
+        QString("&nbsp; 2010-2013 <a href=\"mailto:tka4ev@gmail.com\">Alexandr Tkachev</a><br/>")+
+        tr("&nbsp;&nbsp;&nbsp; (developer since version 2.0.3)<br/>")+
         QString("<br/>")+
-        QString("&nbsp; Uladzimir Bely &lt;wiselord1983@gmail.com&gt;<br/>")+
-        tr("&nbsp; (version 0.4.10 and later)<br/>")
+        tr("<b>Graphic files</b><br/>")+
+        QString("<br/>")+
+        QString("&nbsp; 2009-2010 <a href=\"mailto:wiselord1983@gmail.com\">Uladzimir Bely</a><br/>")+
+        tr("&nbsp;&nbsp;&nbsp; (creator of the logo of the project)<br/>")+
+        QString("<br/>")
         );
 
+    a.textBrowser_TRANSLATION->document()->setDefaultStyleSheet(html_format);
+
     a.textBrowser_TRANSLATION->setText(
-        tr("&nbsp;<u>Russian translation</u><br/>")+
+        tr("Participate in the translation. It is easy: "
+        "<a href=\"https://www.transifex.com/projects/p/eiskaltdcpp/\">"
+        "https://www.transifex.com/projects/p/eiskaltdcpp/</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Uladzimir Bely &lt;wiselord1983@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 0.4.10 and later)<br/>")+
+        tr("Russian translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2009-2010 <a href=\"mailto:wiselord1983@gmail.com\">Uladzimir Bely</a><br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2013 <a href=\"mailto:tehnick-8@mail.ru\">Boris Pek</a> aka Tehnick<br/>")+
         QString("<br/>")+
-        QString("&nbsp; Boris Pek aka Tehnick &lt;tehnick-8@mail.ru&gt;<br/>")+
-        tr("&nbsp; (for 2.1.0 and later)<br/>")+
+        tr("Belarusian translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2009-2013 <a href=\"mailto:i.kliok@gmail.com\">Paval Shalamitski</a> aka Klyok<br/>")+
         QString("<br/>")+
-        tr("&nbsp;<u>Belarusian translation</u><br/>")+
+        tr("Hungarian translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2012 <a href=\"mailto:husumo@gmail.com\">Akos Berki</a> aka sumo<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2011-2013 <a href=\"mailto:marcus@elitemail.hu\">Márk Lutring</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Paval Shalamitski aka Klyok &lt;i.kliok@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 1.0.40 and later)<br/>")+
+        tr("French translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2012 <a href=\"mailto:alexandre.wallimann@gmail.com\">Alexandre Wallimann</a> aka Ale<br/>")+
         QString("<br/>")+
-        tr("&nbsp;<u>Hungarian translation</u><br/>")+
+        tr("Polish translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2012 <a href=\"mailto:arahael@gmail.com\">Arahael</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Akos Berki aka sumo &lt;husumo@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.0.1 and later)<br/>")+
+        tr("Ukrainian translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010 <a href=\"mailto:dmytro.demenko@gmail.com\">Dmytro Demenko</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Márk Lutring &lt;marcus@elitemail.hu&gt;<br/>")+
-        tr("&nbsp; (for 2.2.5 and later)<br/>")+
+        tr("Serbian (Latin) translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2013 <a href=\"mailto:miroslav031@gmail.com\">Miroslav Petrovic</a><br/>")+
         QString("<br/>")+
-        tr("&nbsp;<u>French translation</u><br/>")+
+        tr("Spanish translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2013 <a href=\"mailto:sl1pkn07@gmail.com\">Gustavo Alvarez</a> aka sL1pKn07<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2012 <a href=\"mailto:klondike at klondike.es\">Francisco Blas Izquierdo Riera</a> aka klondike<br/>")+
         QString("<br/>")+
-        QString("&nbsp; Alexandre Wallimann aka Ale &lt;alexandre.wallimann@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.0.2 and later)<br/>")+
+        tr("Bulgarian translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2012 <a href=\"mailto:dimitrov.rusi@gmail.com\">Rusi Dimitrov</a> aka PsyTrip<br/>")+
         QString("<br/>")+
-        tr("&nbsp;<u>Polish translation</u><br/>")+
+        tr("Slovak translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2010-2012 <a href=\"mailto:martin.durisin@gmail.com\">Martin Durisin</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Arahael &lt;arahael@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.0.2 and later)<br/>")+
+        tr("Czech translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2011-2012 <a href=\"mailto:uhlikx@seznam.cz\">Uhlik</a><br/>")+
         QString("<br/>")+
-        tr("&nbsp;<u>Ukrainian translation</u><br/>")+
+        tr("German translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2011-2012 <a href=\"mailto:kgeorgokitsos@yahoo.de\">Konstantinos Georgokitsos</a><br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2011-2012 <a href=\"mailto:tilkax@gmail.com\">Tillmann Karras</a><br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2012-2013 <a href=\"mailto:be.w@mail.ru\">Benjamin Weber</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Dmytro Demenko &lt;dmytro.demenko@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.0.3 and later)<br/>")+
+        tr("Greek translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2011-2012 <a href=\"mailto:kgeorgokitsos@yahoo.de\">Konstantinos Georgokitsos</a><br/>")+
         QString("<br/>")+
-        tr("&nbsp;<u>Serbian (Latin) translation</u><br/>")+
+        tr("Italian translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2012 <a href=\"mailto:netcelli@gmail.com\">Stefano Simoncelli</a><br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2012 <a href=\"mailto:lorenzo.keller@gmail.com\">Lorenzo Keller</a><br/>")+
         QString("<br/>")+
-        QString("&nbsp; Miroslav Petrovic &lt;miroslav031@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.0.3 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>Spanish translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Gustavo Alvarez aka sL1pKn07 &lt;sl1pkn07@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.1.0 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>Bulgarian translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Rusi Dimitrov aka PsyTrip &lt;dimitrov.rusi@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.1.0 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>Slovak translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Martin Durisin &lt;martin.durisin@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.1.0 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>Czech translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Uhlik &lt;uhlikx@seznam.cz&gt;<br/>")+
-        tr("&nbsp; (for 2.2.0 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>German translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Konstantinos Georgokitsos &lt;kgeorgokitsos@yahoo.de&gt;<br/>")+
-        tr("&nbsp; (for 2.2.5 and later)<br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Tillmann Karras &lt;tilkax@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.2.5 and later)<br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Benjamin Weber &lt;be.w@mail.ru&gt;<br/>")+
-        tr("&nbsp; (for 2.2.6 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>Greek translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Konstantinos Georgokitsos &lt;kgeorgokitsos@yahoo.de&gt;<br/>")+
-        tr("&nbsp; (for 2.2.5 and later)<br/>")+
-        QString("<br/>")+
-        tr("&nbsp;<u>Italian translation</u><br/>")+
-        QString("<br/>")+
-        QString("&nbsp; Stefano Simoncelli &lt;netcelli@gmail.com&gt;<br/>")+
-        tr("&nbsp; (for 2.2.6 and later)<br/>")
+        tr("Portuguese (Brazil) translation<br/>")+
+        QString("&nbsp;&nbsp;&nbsp; 2013 <a href=\"mailto:heldercro@gmail.com\">Helder Cesar</a> aka redrum<br/>")+
+        QString("<br/>")
+        );
+
+    a.textBrowser_LICENSE->document()->setDefaultStyleSheet(html_format);
+
+    a.textBrowser_LICENSE->setText(
+        QString("This program is free software: you can redistribute it and/or modify it under the terms "
+                "of the GNU General Public License as published by the Free Software Foundation, either "
+                "version 3 of the License, or (at your option) any later version.<br/>"
+                "<br/>"
+                "This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; "
+                "without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. "
+                "See the GNU General Public License for more details.<br/>"
+                "<br/>"
+                "You should have received a copy of the GNU General Public License along with this program. "
+                "If not, see &lt;<a href=\"http://www.gnu.org/licenses/\">http://www.gnu.org/licenses/</a>&gt;.<br/>")
         );
 
     a.exec();
@@ -2729,16 +2751,16 @@ void MainWindow::slotUnixSignal(int sig){
 
 void MainWindow::slotCloseCurrentWidget(){
     Q_D(MainWindow);
-    
+
     ArenaWidget *awgt = dynamic_cast<ArenaWidget*>(d->arena->widget());
-    
+
     if (awgt)
         ArenaWidgetManager::getInstance()->rem(awgt);
 }
 
 void MainWindow::slotSideBarDockMenu(){
     Q_D(MainWindow);
-    
+
     QMenu *m = new QMenu(this);
     QAction *act = new QAction(tr("Show close buttons"), m);
 
@@ -2764,9 +2786,45 @@ void MainWindow::slotAboutQt(){
     QMessageBox::aboutQt(this);
 }
 
+void MainWindow::slotUpdateFavHubMenu() {
+    Q_D(MainWindow);
+
+    d->favHubMenu->clear();
+
+    const FavoriteHubEntryList& fl = FavoriteManager::getInstance()->getFavoriteHubs();
+
+    for(auto i = fl.cbegin(); i != fl.cend(); ++i) {
+        const FavoriteHubEntry &entry = *(*i);
+
+        QString url = _q(entry.getServer());
+        QString name = entry.getName().empty() ? tr("[No name]") : _q(entry.getName());
+        QString encoding = WulforUtil::getInstance()->dcEnc2QtEnc(QString::fromStdString(entry.getEncoding()));
+        QString menuItem = QString("%1 - %2").arg(name).arg(url);
+
+        QAction *action = new QAction(menuItem, d->favHubMenu);
+        action->setStatusTip(encoding);
+        action->setToolTip(url);
+
+        if (qobject_cast<HubFrame*>(HubManager::getInstance()->getHub(url))) {
+            action->setCheckable(true);
+            action->setChecked(true);
+        }
+
+        d->favHubMenu->addAction(action);
+    }
+}
+
+void MainWindow::slotConnectFavHub(QAction *action) {
+
+    QString url = action->toolTip();
+    QString encoding = action->statusTip();
+
+    newHubFrame(url, encoding);
+}
+
 void MainWindow::nextMsg(){
     Q_D(MainWindow);
-    
+
     HubFrame *fr = qobject_cast<HubFrame*>(HubManager::getInstance()->activeHub());
 
     if (fr)
