@@ -87,6 +87,11 @@ public:
     bool isHash;
     bool stop;
     int left_pane_old_size;
+
+    QString target;
+    uint64_t searchStartTime;
+    uint64_t searchEndTime;
+    bool waitingResults;
 };
 
 QVariant SearchStringListModel::data(const QModelIndex &index, int role) const{
@@ -371,6 +376,9 @@ SearchFrame::SearchFrame(QWidget *parent): QWidget(parent), d_ptr(new SearchFram
     d->completer = NULL;
     d->stop = false;
     d->arena_title = tr("Search");
+    d->searchStartTime = 0;
+    d->searchEndTime = 1;
+    d->waitingResults = false;
 
     setupUi(this);
 
@@ -378,7 +386,11 @@ SearchFrame::SearchFrame(QWidget *parent): QWidget(parent), d_ptr(new SearchFram
 
     ClientManager* clientMgr = ClientManager::getInstance();
 
+#ifdef DO_NOT_USE_MUTEX
     clientMgr->lock();
+#else // DO_NOT_USE_MUTEX
+    auto lock = clientMgr->lock();
+#endif // DO_NOT_USE_MUTEX
     clientMgr->addListener(this);
     Client::List& clients = clientMgr->getClients();
 
@@ -392,10 +404,13 @@ SearchFrame::SearchFrame(QWidget *parent): QWidget(parent), d_ptr(new SearchFram
         d->client_list.push_back(client);
     }
 
+#ifdef DO_NOT_USE_MUTEX
     clientMgr->unlock();
+#endif // DO_NOT_USE_MUTEX
 
     d->str_model->setStringList(d->hubs);
 
+
     for (int i = 0; i < d->str_model->rowCount(); i++)
        d-> str_model->setData(d->str_model->index(i, 0), Qt::Checked, Qt::CheckStateRole);
 
@@ -406,12 +421,12 @@ SearchFrame::~SearchFrame(){
     Menu::deleteInstance();
 
     Q_D(SearchFrame);
-    
+
     treeView_RESULTS->setModel(NULL);
 
     if (d->completer)
         d->completer->deleteLater();
-    
+
     if (d->proxy)
         d->proxy->deleteLater();
 
@@ -428,14 +443,14 @@ void SearchFrame::closeEvent(QCloseEvent *e){
     ClientManager::getInstance()->removeListener(this);
 
     Q_D(SearchFrame);
-    
+
     if (d->timer)
         d->timer->stop();
-    
+
     save();
 
     setAttribute(Qt::WA_DeleteOnClose);
-    
+
     QWidget::disconnect(this, NULL, this, NULL);
 
     e->accept();
@@ -545,7 +560,7 @@ void SearchFrame::init(){
     connect(GlobalTimer::getInstance(), SIGNAL(second()), this, SLOT(slotTimer()));
     connect(pushButton_SIDEPANEL, SIGNAL(clicked()), this, SLOT(slotToggleSidePanel()));
     connect(lineEdit_SEARCHSTR, SIGNAL(returnPressed()), this, SLOT(slotStartSearch()));
-	connect(lineEdit_SIZE,      SIGNAL(returnPressed()), this, SLOT(slotStartSearch()));
+    connect(lineEdit_SIZE,      SIGNAL(returnPressed()), this, SLOT(slotStartSearch()));
     connect(comboBox_FILETYPES, SIGNAL(currentIndexChanged(int)), lineEdit_SEARCHSTR, SLOT(setFocus()));
     connect(comboBox_FILETYPES, SIGNAL(currentIndexChanged(int)), lineEdit_SEARCHSTR, SLOT(selectAll()));
     connect(toolButton_CLOSEFILTER, SIGNAL(clicked()), this, SLOT(slotFilter()));
@@ -736,21 +751,8 @@ void SearchFrame::removeSource(const VarMap &params){
 
 }
 
-void SearchFrame::timerTick(){
-    ////int32_t waitFor = SearchManager::getInstance()->timeToSearch();
+void SearchFrame::timerTick() {
 
-    //if (waitFor > 0){
-        //QString msg = tr("Searching too soon, next search in %1 second").arg(waitFor);
-
-        //status->setText(msg);
-
-        //arena_title = tr("Search - %1").arg(msg);
-
-        //timer->start();
-    //}
-    //else {
-        //status->setText(tr("Ready to search..."));
-    //}
 }
 
 void SearchFrame::onHubAdded(const QString &info){
@@ -1073,7 +1075,14 @@ void SearchFrame::slotStartSearch(){
         ftype = SearchManager::TYPE_ANY;
     }
 
-    SearchManager::getInstance()->search(clients, s.toStdString(), llsize, (SearchManager::TypeModes)ftype, searchMode, d->token.toStdString(), exts);
+    d->target = s;
+    d->searchStartTime = GlobalTimer::getInstance()->getTicks()*1000;
+
+    uint64_t maxDelayBeforeSearch = SearchManager::getInstance()->search(clients, s.toStdString(), llsize, (SearchManager::TypeModes)ftype, searchMode, d->token.toStdString(), exts, (void*)this);
+    uint64_t waitingResultsTime = 20000; // just assumption that user receives most of results in 20 seconds
+
+    d->searchEndTime = d->searchStartTime + maxDelayBeforeSearch + waitingResultsTime;
+    d->waitingResults = true;
 
     if (!checkBox_HIDEPANEL->isChecked()){
         QList<int> panes = splitter->sizes();
@@ -1485,7 +1494,7 @@ void SearchFrame::slotContextMenu(const QPoint &){
 
                 StringMap params;
 
-                if (WulforUtil::getInstance()->getUserCommandParams(last_user_cmd, params)){
+                if (WulforUtil::getInstance()->getUserCommandParams(uc, params)){
                     UserPtr user = ClientManager::getInstance()->findUser(CID(item->cid.toStdString()));
 
                     if (user && user->isOnline()){
@@ -1552,12 +1561,28 @@ void SearchFrame::slotHeaderMenu(const QPoint&){
 void SearchFrame::slotTimer(){
     Q_D(SearchFrame);
 
+    if (d->waitingResults) {
+        uint64_t now = GlobalTimer::getInstance()->getTicks()*1000;
+        float fraction  = 100.0f*(now - d->searchStartTime)/(d->searchEndTime - d->searchStartTime);
+        if (fraction >= 100.0) {
+            fraction = 100.0;
+            d->waitingResults = false;
+        }
+        QString msg = tr("Searching for %1 ...").arg(d->target);
+        progressBar->setFormat(msg);
+        progressBar->setValue(static_cast<unsigned>(fraction));
+    } else {
+        QString msg = "";
+        progressBar->setFormat(msg);
+        progressBar->setValue(0);
+    }
+
     if (d->dropped == d->results && d->dropped == 0){
 
         if (d->currentSearch.empty())
-            status->hide();
+            frame_PROGRESS->hide();
         else {
-            status->show();
+            frame_PROGRESS->show();
 
             QString text = QString(tr("<b>No results</b>"));
 
@@ -1565,8 +1590,8 @@ void SearchFrame::slotTimer(){
         }
     }
     else {
-        if (!status->isVisible())
-            status->show();
+        if (!frame_PROGRESS->isVisible())
+            frame_PROGRESS->show();
 
         QString text = QString(tr("Found: <b>%1</b>  Dropped: <b>%2</b>")).arg(d->results).arg(d->dropped);
 
@@ -1706,6 +1731,7 @@ void SearchFrame::on(ClientUpdated, Client* c) noexcept{
 void SearchFrame::on(ClientDisconnected, Client* c) noexcept{
     emit coreClientDisconnected((_q(c->getHubUrl())));
 }
+
 void SearchFrame::slotStopSearch(){
     Q_D(SearchFrame);
 
